@@ -46,6 +46,7 @@
 #include "feedstorage.h"
 #include "storage.h"
 #include "treenodevisitor.h"
+#include "utils.h"
 
 #include "librss/librss.h"
 
@@ -62,9 +63,14 @@ class Feed::FeedPrivate
         bool markImmediatelyAsRead;
         bool useNotification;
         bool loadLinkedWebsite;
-        int lastFetched;
 
         bool fetchError;
+        
+        int lastErrorFetch; // save time of last fetch that went wrong.
+                            // != lastFetch property from the archive 
+                            // (that saves the last _successfull fetch!)
+                            // workaround for 3.5.x
+
         int fetchTries;
         bool followDiscovery;
         RSS::Loader* loader;
@@ -157,7 +163,7 @@ Feed* Feed::fromOPML(QDomElement e)
         feed->setMarkImmediatelyAsRead(markImmediatelyAsRead);
         feed->setLoadLinkedWebsite(loadLinkedWebsite);
         feed->loadArticles(); // TODO: make me fly: make this delayed
-        
+        feed->loadImage();
     }
 
     return feed;
@@ -198,6 +204,14 @@ QValueList<Article> Feed::articles(const QString& tag)
     }
 }
 
+void Feed::loadImage()
+{
+    QString imageFileName = KGlobal::dirs()->saveLocation("cache", "akregator/Media/") 
+                            + Utils::fileNameForUrl(d->xmlUrl) + 
+".png";
+    d->imagePixmap.load(imageFileName, "PNG");
+}
+        
 void Feed::loadArticles()
 {
     if (d->articlesLoaded)
@@ -267,6 +281,7 @@ Feed::Feed() : TreeNode(), d(new FeedPrivate)
     d->markImmediatelyAsRead = false;
     d->useNotification = false;
     d->fetchError = false;
+    d->lastErrorFetch = 0;
     d->fetchTries = 0;
     d->loader = 0;
     d->articlesLoaded = false;
@@ -395,6 +410,14 @@ void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
         queue->addFeed(this);
     else
     {
+        uint now = QDateTime::currentDateTime().toTime_t();
+
+        // workaround for 3.5.x: if the last fetch went wrong, try again after 30 minutes
+        // this fixes annoying behaviour of akregator, especially when the host is reachable
+        // but Akregator can't parse the feed (the host is hammered every minute then)
+        if ( fetchErrorOccurred() && now - d->lastErrorFetch <= 30*60 )
+             return;
+
         int interval = -1;
 
         if (useCustomFetchInterval() )
@@ -405,8 +428,6 @@ void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
 
         uint lastFetch = d->archive->lastFetch();
 
-        uint now = QDateTime::currentDateTime().toTime_t();
-
         if ( interval > 0 && now - lastFetch >= (uint)interval )
             queue->addFeed(this);
     }
@@ -455,12 +476,14 @@ void Feed::appendArticles(const RSS::Document &doc)
             if (!mya.guidIsHash() && mya.hash() != old.hash() && !old.isDeleted())
             {
                 mya.setKeep(old.keep());
+                int oldstatus = old.status();
                 old.setStatus(Article::Read);
+
                 d->articles.remove(old.guid());
                 appendArticle(mya);
-                // reset status to New
-                if (!mya.isDeleted() && !markImmediatelyAsRead())
-                    mya.setStatus(Article::New);
+
+                mya.setStatus(oldstatus);
+
                 d->updatedArticlesNotify.append(mya);
                 changed = true;
             }
@@ -561,7 +584,13 @@ void Feed::tryFetch()
 
 void Feed::slotImageFetched(const QPixmap& image)
 {
-    setImage(image);
+    if (image.isNull())
+        return;
+    d->imagePixmap=image;
+    d->imagePixmap.save(KGlobal::dirs()->saveLocation("cache", "akregator/Media/") 
+                        + Utils::fileNameForUrl(d->xmlUrl) + 
+".png","PNG");
+    nodeModified();
 }
 
 void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
@@ -587,6 +616,7 @@ void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
         else
         {
             d->fetchError = true;
+            d->lastErrorFetch = QDateTime::currentDateTime().toTime_t();
             emit fetchError(this);
         }
         return;
@@ -600,19 +630,11 @@ void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
 
     d->fetchError = false;
     
-    if (d->imagePixmap.isNull())
+    if (doc.image() && d->imagePixmap.isNull())
     {
-        QString u = d->xmlUrl;
-        QString imageFileName = KGlobal::dirs()->saveLocation("cache", "akregator/Media/")+u.replace("/", "_").replace(":", "_")+".png";
-        d->imagePixmap=QPixmap(imageFileName, "PNG");
-
-        // if we aint got teh image and the feed provides one, get it....
-        if (d->imagePixmap.isNull() && doc.image())
-        {
-            d->image = *doc.image();
-            connect(&d->image, SIGNAL(gotPixmap(const QPixmap&)), this, SLOT(slotImageFetched(const QPixmap&)));
-            d->image.getPixmap();
-        }   
+        d->image = *doc.image();
+        connect(&d->image, SIGNAL(gotPixmap(const QPixmap&)), this, SLOT(slotImageFetched(const QPixmap&)));
+        d->image.getPixmap();
     }
 
     if (title().isEmpty())
@@ -675,16 +697,6 @@ void Feed::setFavicon(const QPixmap &p)
     nodeModified();
 }
 
-void Feed::setImage(const QPixmap &p)
-{
-    if (p.isNull())
-        return;
-    d->imagePixmap=p;
-    QString u = d->xmlUrl;
-    d->imagePixmap.save(KGlobal::dirs()->saveLocation("cache", "akregator/Media/")+u.replace("/", "_").replace(":", "_")+".png","PNG");
-    nodeModified();
-}
-
 Feed::ArchiveMode Feed::archiveMode() const
 {
     return d->archiveMode;
