@@ -1,7 +1,7 @@
 /*
  *  kamail.cpp  -  email functions
  *  Program:  kalarm
- *  Copyright (c) 2002 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2002-2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -168,7 +168,7 @@ bool KAMail::send(const KAEvent& event, QStringList& errmsgs, bool allowNotify)
 				command += KShellProcess::quote(data.bcc);
 			}
 
-			command += " ";
+			command += ' ';
 			command += event.emailAddresses(" "); // locally provided, okay
 		}
 
@@ -954,7 +954,7 @@ bool parseUserName( const char* & scursor, const char * const send,
   QString maybeLocalPart;
   QString tmp;
 
-  while ( scursor != send ) {
+  if ( scursor != send ) {
     // first, eat any whitespace
     eatCFWS( scursor, send, isCRLF );
 
