@@ -1,7 +1,7 @@
 /*
  *  mainwindow.h  -  main application window
  *  Program:  kalarm
- *  Copyright (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2006 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ class MainWindow : public MainWindowBase
 		                                      { executeNew(w, 0, a, t); }
 		static void        executeNew(const KAEvent& e, MainWindow* w = 0)
 		                                      { executeNew(w, &e); }
+		static void        executeEdit(KAEvent&, MainWindow* = 0);
 		static void        executeDragEnterEvent(QDragEnterEvent*);
 		static void        executeDropEvent(MainWindow*, QDropEvent*);
 		static void        closeAll();
@@ -103,12 +104,14 @@ class MainWindow : public MainWindowBase
 		void           slotEnable();
 		void           slotToggleTrayIcon();
 		void           slotResetDaemon();
+		void           slotImportAlarms();
 		void           slotBirthdays();
 		void           slotTemplates();
 		void           slotTemplatesEnd();
 		void           slotPreferences();
 		void           slotConfigureKeys();
 		void           slotConfigureToolbar();
+		void           slotNewToolbarConfig();
 		void           slotQuit();
 		void           slotDeletion();
 		void           slotSelection();
@@ -128,6 +131,7 @@ class MainWindow : public MainWindowBase
 		void           slotFindActive(bool);
 		void           updateTrayIconAction();
 		void           updateActionsMenu();
+		void           columnsReordered();
 
 	private:
 		typedef QValueList<MainWindow*> WindowList;
