@@ -30,6 +30,7 @@
 #include "task.h"
 #include "taskview.h"
 #include "timekard.h"
+#include "taskviewwhatsthis.h"
 
 #define T_LINESIZE 1023
 #define HIDDEN_COLUMN -10
@@ -41,9 +42,6 @@ TaskView::TaskView(QWidget *parent, const char *name, const QString &icsfile ):K
   _preferences = Preferences::instance( icsfile );
   _storage = KarmStorage::instance();
 
-  connect(this, SIGNAL( doubleClicked( QListViewItem * )),
-          this, SLOT( changeTimer( QListViewItem * )));
-
   connect( this, SIGNAL( expanded( QListViewItem * ) ),
            this, SLOT( itemStateChanged( QListViewItem * ) ) );
   connect( this, SIGNAL( collapsed( QListViewItem * ) ),
@@ -83,8 +81,8 @@ TaskView::TaskView(QWidget *parent, const char *name, const QString &icsfile ):K
   _idleTimeDetector = new IdleTimeDetector( _preferences->idlenessTimeout() );
   connect( _idleTimeDetector, SIGNAL( extractTime(int) ),
            this, SLOT( extractTime(int) ));
-  connect( _idleTimeDetector, SIGNAL( stopAllTimers() ),
-           this, SLOT( stopAllTimers() ));
+  connect( _idleTimeDetector, SIGNAL( stopAllTimersAt(QDateTime) ),
+           this, SLOT( stopAllTimersAt(QDateTime) ));
   connect( _preferences, SIGNAL( idlenessTimeout(int) ),
            _idleTimeDetector, SLOT( setMaxIdle(int) ));
   connect( _preferences, SIGNAL( detectIdleness(bool) ),
@@ -110,6 +108,7 @@ TaskView::TaskView(QWidget *parent, const char *name, const QString &icsfile ):K
            this, SLOT( startTimerFor(Task*) ));
   connect( _desktopTracker, SIGNAL( leftActiveDesktop( Task* ) ),
            this, SLOT( stopTimerFor(Task*) ));
+  new TaskViewWhatsThis( this );
 }
 
 KarmStorage* TaskView::storage()
@@ -117,6 +116,53 @@ KarmStorage* TaskView::storage()
   return _storage;
 }
 
+void TaskView::contentsMousePressEvent ( QMouseEvent * e )
+{
+  kdDebug(5970) << "entering contentsMousePressEvent" << endl;
+  KListView::contentsMousePressEvent(e);
+  Task* task = current_item();
+  // This checks that there has been a click onto an item,  
+  // not into an empty part of the KListView.
+  if ( task != 0 &&  // zero can happen if there is no task
+        e->pos().y() >= current_item()->itemPos() &&
+        e->pos().y() < current_item()->itemPos()+current_item()->height() )
+  { 
+    // see if the click was on the completed icon
+    int leftborder = treeStepSize() * ( task->depth() + ( rootIsDecorated() ? 1 : 0)) + itemMargin();
+    if ((leftborder < e->x()) && (e->x() < 19 + leftborder ))
+    {
+      if ( task->isComplete() ) task->setPercentComplete( 0, _storage );
+      else task->setPercentComplete( 100, _storage );
+    }
+    emit updateButtons();
+  }
+}
+
+void TaskView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
+// if the user double-clicks onto a tasks, he says "I am now working exclusively
+// on that task". That means, on a doubleclick, we check if it occurs on an item
+// not in the blank space, if yes, stop all other tasks and start the new timer.
+{
+  kdDebug(5970) << "entering contentsMouseDoubleClickEvent" << endl;
+  KListView::contentsMouseDoubleClickEvent(e);
+  
+  Task *task = current_item();
+
+  if ( task != 0 )  // current_item() exists
+  {
+    if ( e->pos().y() >= task->itemPos() &&  // doubleclick was onto current_item()
+          e->pos().y() < task->itemPos()+task->height() )
+    {
+      if ( activeTasks.findRef(task) == -1 )  // task is active
+      {
+        stopAllTimers();
+        startCurrentTimer();
+      }
+      else stopCurrentTimer();
+    }
+  }
+}
+
 TaskView::~TaskView()
 {
   _preferences->save();
@@ -137,10 +183,12 @@ Task* TaskView::item_at_index(int i)
   return static_cast<Task*>(itemAtIndex(i));
 }
 
-void TaskView::load()
+void TaskView::load( QString fileName )
 {
+  // if the program is used as an embedded plugin for konqueror, there may be a need
+  // to load from a file without touching the preferences.
   _isloading = true;
-  QString err = _storage->load(this, _preferences);
+  QString err = _storage->load(this, _preferences, fileName);
 
   if (!err.isEmpty())
   {
@@ -160,6 +208,7 @@ void TaskView::load()
   setCurrentItem(first_child());
   _desktopTracker->startTracking();
   _isloading = false;
+  refresh();
 }
 
 void TaskView::restoreItemState( QListViewItem *item )
@@ -188,7 +237,6 @@ void TaskView::closeStorage() { _storage->closeStorage( this ); }
 void TaskView::iCalFileModified(ResourceCalendar *rc)
 {
   kdDebug(5970) << "entering iCalFileModified" << endl;
-  stopAllTimers();
   kdDebug(5970) << rc->infoText() << endl;
   rc->dump();
   _storage->buildTaskView(rc,this);
@@ -202,16 +250,7 @@ void TaskView::refresh()
   int i = 0;
   for ( Task* t = item_at_index(i); t; t = item_at_index(++i) )
   {
-    if (!t->isComplete())
-    {
-      t->setOpen(true);
-      t->setEnabled(true);
-    }
-    else
-    {
-      t->setOpen(false);
-      t->setEnabled(false);
-    }
+    t->setPixmapProgress();
   }
   
   // remove root decoration if there is no more children.
@@ -227,6 +266,7 @@ void TaskView::refresh()
   if (!anyChilds) {
     setRootIsDecorated(false);
   }
+  emit updateButtons();
   kdDebug(5970) << "exiting TaskView::refresh()" << endl;
 }
     
@@ -262,19 +302,23 @@ void TaskView::loadFromFlatFile()
   }
 }
 
-void TaskView::importPlanner()
+QString TaskView::importPlanner(QString fileName)
 {
   kdDebug(5970) << "entering importPlanner" << endl;
-  {
-    PlannerParser* handler=new PlannerParser(this);
-    QFile xmlFile(KFileDialog::getOpenFileName(QString::null, QString::null, 0) );
-    QXmlInputSource source( xmlFile );
-    QXmlSimpleReader reader;
-    reader.setContentHandler( handler );
-    reader.parse( source );
-    // we need a refresh
-    refresh();
-  }
+  PlannerParser* handler=new PlannerParser(this);
+  if (fileName.isEmpty()) fileName=KFileDialog::getOpenFileName(QString::null, QString::null, 0);
+  QFile xmlFile( fileName );
+  QXmlInputSource source( xmlFile );
+  QXmlSimpleReader reader;
+  reader.setContentHandler( handler );
+  reader.parse( source );
+  refresh();
+  return "";
+}
+
+QString TaskView::report( const ReportCriteria& rc )
+{
+  return _storage->report( this, rc );
 }
 
 void TaskView::exportcsvFile()
@@ -308,38 +352,20 @@ QString TaskView::exportcsvHistory()
 
 void TaskView::scheduleSave()
 {
-    _manualSaveTimer->start( 10, true /*single-shot*/ );
+  kdDebug(5970) << "Entering TaskView::scheduleSave" << endl;
+  // save changes a little while after they happen
+  _manualSaveTimer->start( 10, true /*single-shot*/ );
 }
 
 Preferences* TaskView::preferences() { return _preferences; }
 
 QString TaskView::save()
+// This saves the not-running tasks.
 {
-    // DF: this code created a new event for the running task(s),
-    // at every call (very frequent with autosave) !!!
-    // -> if one wants autosave to save the current event, then
-    // Task needs to store the "current event" and we need to update
-    // it before calling save.
-#if 0
-  // Stop then start all timers so history entries are written.  This is
-  // inefficient if more than one task running, but it is correct.  It is
-  // inefficient because the iCalendar file is saved every time a task's
-  // setRunning(false, ...) is called.  For a big ics file, this could be a
-  // drag.  However, it does ensure that the data will be consistent.  And
-  // if the most common use case is that one task is running most of the time,
-  // it won't make any difference.
-  for (unsigned int i = 0; i < activeTasks.count(); i++)
-  {
-    activeTasks.at(i)->setRunning(false, _storage);
-    activeTasks.at(i)->setRunning(true, _storage);
-  }
-
-  // If there was an active task, the iCal file has already been saved.
-  if (activeTasks.count() == 0)
-#endif
-  {
-    return _storage->save(this);
-  }
+  kdDebug(5970) << "Entering TaskView::save" << endl;
+  QString err = _storage->save(this);
+  emit(setStatusBar(err));
+  return err;
 }
 
 void TaskView::startCurrentTimer()
@@ -354,11 +380,11 @@ long TaskView::count()
   return n;
 }
 
-void TaskView::startTimerFor(Task* task)
+void TaskView::startTimerFor(Task* task, QDateTime startTime )
 {
   if (task != 0 && activeTasks.findRef(task) == -1) {
     _idleTimeDetector->startIdleDetection();
-    task->setRunning(true, _storage);
+    task->setRunning(true, _storage, startTime);
     activeTasks.append(task);
     emit updateButtons();
     if ( activeTasks.count() == 1 )
@@ -368,8 +394,14 @@ void TaskView::startTimerFor(Task* task)
   }
 }
 
+void TaskView::clearActiveTasks()
+{
+  activeTasks.clear();
+}
+
 void TaskView::stopAllTimers()
 {
+  kdDebug(5970) << "Entering TaskView::stopAllTimers()" << endl;
   for ( unsigned int i = 0; i < activeTasks.count(); i++ )
     activeTasks.at(i)->setRunning(false, _storage);
 
@@ -377,7 +409,25 @@ void TaskView::stopAllTimers()
   activeTasks.clear();
   emit updateButtons();
   emit timersInactive();
-  emit tasksChanged( activeTasks);
+  emit tasksChanged( activeTasks );
+}
+
+void TaskView::stopAllTimersAt(QDateTime qdt)
+// stops all timers for the time qdt. This makes sense, if the idletimedetector detected
+// the last work has been done 50 minutes ago.
+{
+  kdDebug(5970) << "Entering TaskView::stopAllTimersAt" << endl;
+  for ( unsigned int i = 0; i < activeTasks.count(); i++ )
+  {
+    activeTasks.at(i)->setRunning(false, _storage, qdt, qdt);
+    kdDebug() << activeTasks.at(i)->name() << endl;
+  }
+
+  _idleTimeDetector->stopIdleDetection();
+  activeTasks.clear();
+  emit updateButtons();
+  emit timersInactive();
+  emit tasksChanged( activeTasks );
 }
 
 void TaskView::startNewSession()
@@ -400,6 +450,7 @@ void TaskView::resetTimeForAllTasks()
 
 void TaskView::stopTimerFor(Task* task)
 {
+  kdDebug(5970) << "Entering stopTimerFor. task = " << task->name() << endl;
   if ( task != 0 && activeTasks.findRef(task) != -1 ) {
     activeTasks.removeRef(task);
     task->setRunning(false, _storage);
@@ -417,24 +468,6 @@ void TaskView::stopCurrentTimer()
   stopTimerFor( current_item());
 }
 
-
-void TaskView::changeTimer(QListViewItem *)
-{
-  Task *task = current_item();
-
-  if ( task != 0 && activeTasks.findRef(task) == -1 )
-  {
-    // Stop all the other timers.
-    for (unsigned int i=0; i<activeTasks.count();i++)
-      (activeTasks.at(i))->setRunning(false, _storage);
-    activeTasks.clear();
-
-    // Start the new timer.
-    startCurrentTimer();
-  }
-  else stopCurrentTimer();
-}
-
 void TaskView::minuteUpdate()
 {
   addTimeToActiveTasks(1, false);
@@ -474,7 +507,7 @@ void TaskView::newTask(QString caption, Task *parent)
     if ( uid.isNull() )
     {
       KMessageBox::error( 0, i18n(
-            "Error storing new task. Your changes were not saved." ) );
+            "Error storing new task. Your changes were not saved. Make sure you can edit your iCalendar file. Also quit all applications using this file and remove any lock file related to its name from ~/.kde/share/apps/kabc/lock/ " ) );
     }
 
     delete dialog;
@@ -482,31 +515,30 @@ void TaskView::newTask(QString caption, Task *parent)
 }
 
 QString TaskView::addTask
-( const QString& taskname, long total, long session, const DesktopList& desktops, Task* parent )
+( const QString& taskname, long total, long session, 
+  const DesktopList& desktops, Task* parent )
 {
   Task *task;
+  kdDebug(5970) << "TaskView::addTask: taskname = " << taskname << endl;
 
   if ( parent ) task = new Task( taskname, total, session, desktops, parent );
   else          task = new Task( taskname, total, session, desktops, this );
 
   task->setUid( _storage->addTask( task, parent ) );
-
-  if ( ! task->uid().isNull() )
+  QString taskuid=task->uid();
+  if ( ! taskuid.isNull() )
   {
     _desktopTracker->registerForDesktops( task, desktops );
-
     setCurrentItem( task );
-
     setSelected( task, true );
-
+    task->setPixmapProgress();
     save();
   }
   else
   {
     delete task;
   }
-
-  return task->uid();
+  return taskuid;
 }
 
 void TaskView::newSubTask()
@@ -576,6 +608,23 @@ void TaskView::editTask()
 //    task->addComment( comment, _storage );
 //}
 
+void TaskView::reinstateTask(int completion)
+{
+  Task* task = current_item();
+  if (task == 0) {
+    KMessageBox::information(0,i18n("No task selected."));
+    return;
+  }
+
+  if (completion<0) completion=0;
+  if (completion<100)
+  {
+    task->setPercentComplete(completion, _storage);
+    task->setPixmapProgress();
+    save();
+    emit updateButtons();
+  }
+}
 
 void TaskView::deleteTask(bool markingascomplete)
 {
@@ -585,31 +634,33 @@ void TaskView::deleteTask(bool markingascomplete)
     return;
   }
 
-  int response = KMessageBox::Yes;
+  int response = KMessageBox::Continue;
   if (!markingascomplete && _preferences->promptDelete()) {
     if (task->childCount() == 0) {
-      response = KMessageBox::warningYesNo( 0,
+      response = KMessageBox::warningContinueCancel( 0,
           i18n( "Are you sure you want to delete "
           "the task named\n\"%1\" and its entire history?")
           .arg(task->name()),
-          i18n( "Deleting Task"));
+          i18n( "Deleting Task"), KStdGuiItem::del());
     }
     else {
-      response = KMessageBox::warningYesNo( 0,
+      response = KMessageBox::warningContinueCancel( 0,
           i18n( "Are you sure you want to delete the task named"
           "\n\"%1\" and its entire history?\n"
           "NOTE: all its subtasks and their history will also "
           "be deleted.").arg(task->name()),
-          i18n( "Deleting Task"));
+          i18n( "Deleting Task"), KStdGuiItem::del());
     }
   }
 
-  if (response == KMessageBox::Yes)
+  if (response == KMessageBox::Continue)
   {
     if (markingascomplete)
     {
       task->setPercentComplete(100, _storage);
+      task->setPixmapProgress();
       save();
+      emit updateButtons();
 
       // Have to remove after saving, as the save routine only affects tasks
       // that are in the view.  Otherwise, the new percent complete does not
@@ -619,26 +670,15 @@ void TaskView::deleteTask(bool markingascomplete)
     }
     else
     {
-      QString uid = task->uid();
+      QString uid=task->uid();
       task->remove(activeTasks, _storage);
       task->removeFromView();
-      if ( _preferences ) _preferences->deleteEntry( uid ); // forget if the item was expanded or collapsed
+      if( _preferences ) _preferences->deleteEntry( uid ); // forget if the item was expanded or collapsed
       save();
     }
 
     // remove root decoration if there is no more children.
-    bool anyChilds = false;
-    for(Task* child = first_child();
-              child;
-              child = child->nextSibling()) {
-      if (child->childCount() != 0) {
-        anyChilds = true;
-        break;
-      }
-    }
-    if (!anyChilds) {
-      setRootIsDecorated(false);
-    }
+    refresh();
 
     // Stop idle detection if no more counters are running
     if (activeTasks.count() == 0) {
@@ -652,6 +692,7 @@ void TaskView::deleteTask(bool markingascomplete)
 
 void TaskView::extractTime(int minutes)
 {
+  kdDebug(5970) << "Entering extractTime" << endl;
   addTimeToActiveTasks(-minutes);
 }
 
@@ -710,11 +751,16 @@ void TaskView::deletingTask(Task* deletedTask)
 }
 
 void TaskView::iCalFileChanged(QString file)
+// User might have picked a new file in the preferences dialog.
+// This is not iCalFileModified.
 {
   kdDebug(5970) << "TaskView:iCalFileChanged: " << file << endl;
-  stopAllTimers();
-  _storage->save(this);
-  load();
+  if (_storage->icalfile() != file)
+  {
+    stopAllTimers();
+    _storage->save(this);
+    load();
+  }
 }
 
 QValueList<HistoryEvent> TaskView::getHistory(const QDate& from,
@@ -735,39 +781,74 @@ void TaskView::markTaskAsComplete()
   deleteTask(markingascomplete);
 }
 
+void TaskView::markTaskAsIncomplete()
+{
+  if (current_item())
+    kdDebug(5970) << "TaskView::markTaskAsComplete: "
+      << current_item()->uid() << endl;
+  else
+    kdDebug(5970) << "TaskView::markTaskAsComplete: null current_item()" << endl;
+
+  reinstateTask(50); // if it has been reopened, assume half-done
+}
+
+
 void TaskView::clipTotals()
 {
-  TimeKard *t = new TimeKard();
+  TimeKard t;
   if (current_item() && current_item()->isRoot())
   {
     int response = KMessageBox::questionYesNo( 0,
         i18n("Copy totals for just this task and its subtasks, or copy totals for all tasks?"),
         i18n("Copy Totals to Clipboard"),
         i18n("Copy This Task"), i18n("Copy All Tasks") );
-    if (response == KMessageBox::Yes) // this task only
+    if (response == KMessageBox::Yes) // This task only
     {
-      KApplication::clipboard()->setText(t->totalsAsText(this));
+      KApplication::clipboard()->setText(t.totalsAsText(this, true, TimeKard::TotalTime));
     }
-    else // only task
+    else // All tasks
     {
-      KApplication::clipboard()->setText(t->totalsAsText(this, false));
+      KApplication::clipboard()->setText(t.totalsAsText(this, false, TimeKard::TotalTime));
     }
   }
   else
   {
-    KApplication::clipboard()->setText(t->totalsAsText(this));
+    KApplication::clipboard()->setText(t.totalsAsText(this, true, TimeKard::TotalTime));
+  }
+}
+
+void TaskView::clipSession()
+{
+  TimeKard t;
+  if (current_item() && current_item()->isRoot())
+  {
+    int response = KMessageBox::questionYesNo( 0,
+        i18n("Copy session time for just this task and its subtasks, or copy session time for all tasks?"),
+        i18n("Copy Session Time to Clipboard"),
+        i18n("Copy This Task"), i18n("Copy All Tasks") );
+    if (response == KMessageBox::Yes) // this task only
+    {
+      KApplication::clipboard()->setText(t.totalsAsText(this, true, TimeKard::SessionTime));
+    }
+    else // only task
+    {
+      KApplication::clipboard()->setText(t.totalsAsText(this, false, TimeKard::SessionTime));
+    }
+  }
+  else
+  {
+    KApplication::clipboard()->setText(t.totalsAsText(this, true, TimeKard::SessionTime));
   }
 }
 
 void TaskView::clipHistory()
 {
-
-  PrintDialog *dialog = new PrintDialog();
-  if (dialog->exec()== QDialog::Accepted)
+  PrintDialog dialog;
+  if (dialog.exec()== QDialog::Accepted)
   {
-    TimeKard *t = new TimeKard();
+    TimeKard t;
     KApplication::clipboard()->
-      setText(t->historyAsText(this, dialog->from(), dialog->to()));
+      setText( t.historyAsText(this, dialog.from(), dialog.to(), !dialog.allTasks(), dialog.perWeek(), dialog.totalsOnly() ) );
   }
 }
 
