@@ -409,6 +409,19 @@ void AccountWizard::transportCreated()
   for ( uint i = 0 ; i < mTransportInfoList.count() ; i++ )
     mTransportInfo->writeConfig( i + 1 );
 
+    // No default transport? => set the first transport as the default
+  if ( GlobalSettings::self()->defaultTransport().isEmpty() ) {
+    KConfigGroup general( KMKernel::config(), "General" );
+
+    if ( mTransportInfoList.count() > 0 ) {
+      KMTransportInfo info;
+      info.readConfig( 1 );
+      KConfigGroup composer( KMKernel::config(), "Composer" );
+      GlobalSettings::self()->setDefaultTransport( info.name );
+      GlobalSettings::self()->setCurrentTransport( info.name );
+    }
+  }
+
   mTransportInfoList.setAutoDelete( true );
   mTransportInfoList.clear();
 
