@@ -12,11 +12,13 @@
 #include <qstring.h>
 #include <qvaluelist.h>
 #include <qstringlist.h>
+#include <dcopobject.h>
 
 #include <kdialogbase.h>
 #include <klistview.h>
 #include <kcmodule.h>
 #include <klocale.h>
+#include <kdepimmacros.h>
 
 class QPushButton;
 class QLabel;
@@ -32,7 +34,6 @@ class QListBox;
 class QButtonGroup;
 class QRegExpValidator;
 class QVBox;
-class KMFolderComboBox;
 class KMAccount;
 class KMTransportInfo;
 class ListView;
@@ -44,6 +45,18 @@ class QPoint;
 class ComposerCryptoConfiguration;
 class WarningConfiguration;
 class SMimeConfiguration;
+class TemplatesConfiguration;
+class CustomTemplates;
+class QGroupBox;
+class QVGroupBox;
+#include <qdict.h>
+class QLineEdit;
+class KMMsgTagDesc;
+class KListBox;
+class KColorCombo;
+class KFontRequester;
+class KIconButton;
+class KKeyButton;
 
 namespace Kpgp {
   class Config;
@@ -51,6 +64,8 @@ namespace Kpgp {
 namespace KMail {
   class IdentityDialog;
   class IdentityListView;
+  class AccountComboBox;
+  class FolderRequester;
 }
 namespace Kleo {
   class BackendConfigWidget;
@@ -66,7 +81,7 @@ public:
   enum DuplicateMode { Empty, ControlCenter, ExistingEntry };
 
   NewIdentityDialog( const QStringList & identities,
-		     QWidget *parent=0, const char *name=0, bool modal=true );
+                     QWidget *parent=0, const char *name=0, bool modal=true );
 
   QString identityName() const { return mLineEdit->text(); }
   QString duplicateIdentity() const { return mComboBox->currentText(); }
@@ -92,9 +107,9 @@ struct LanguageItem
 {
   LanguageItem() {}
   LanguageItem( const QString & language, const QString & reply=QString::null,
-		const QString & replyAll=QString::null,
-		const QString & forward=QString::null,
-		const QString & indentPrefix=QString::null ) :
+                const QString & replyAll=QString::null,
+                const QString & forward=QString::null,
+                const QString & indentPrefix=QString::null ) :
     mLanguage( language ), mReply( reply ), mReplyAll( replyAll ),
     mForward( forward ), mIndentPrefix( indentPrefix ) {}
 
@@ -109,7 +124,7 @@ class NewLanguageDialog : public KDialogBase
 
   public:
     NewLanguageDialog( LanguageItemList & suppressedLangs, QWidget *parent=0,
-		       const char *name=0, bool modal=true );
+                       const char *name=0, bool modal=true );
     QString language() const;
 
   private:
@@ -189,16 +204,26 @@ public:
       :QWidget( parent, name )
       {}
   ~ConfigModuleTab() {}
-  virtual void load() = 0;
+  void load();
   virtual void save() = 0;
+  void defaults();
   // the below are optional
-  virtual void defaults() {}
   virtual void installProfile(){}
 signals:
    // forwarded to the ConfigModule
   void changed(bool);
 public slots:
   void slotEmitChanged();
+private:
+  // reimplement this for loading values of settings which are available
+  // via GlobalSettings
+  virtual void doLoadFromGlobalSettings() {}
+  // reimplement this for loading values of settings which are not available
+  // via GlobalSettings
+  virtual void doLoadOther() {}
+  // reimplement this for loading default values of settings which are
+  // not available via GlobalSettings (KConfigXT).
+  virtual void doResetToDefaultsOther() {}
 };
 
 
@@ -212,6 +237,7 @@ public:
   ConfigModuleWithTabs( QWidget * parent=0, const char * name=0 );
    ~ConfigModuleWithTabs() {}
 
+  // don't reimplement any of those methods
   virtual void load();
   virtual void save();
   virtual void defaults();
@@ -232,7 +258,7 @@ private:
 //
 //
 
-class IdentityPage : public ConfigModule {
+class KDE_EXPORT IdentityPage : public ConfigModule {
   Q_OBJECT
 public:
   IdentityPage( QWidget * parent=0, const char * name=0 );
@@ -246,7 +272,7 @@ public:
 public slots:
   void slotUpdateTransportCombo( const QStringList & );
 
-protected slots:
+private slots:
   void slotNewIdentity();
   void slotModifyIdentity();
   void slotRemoveIdentity();
@@ -254,16 +280,16 @@ protected slots:
       KListView::rename on the selected item */
   void slotRenameIdentity();
   /** connected to @p mIdentityList's renamed() signal. Validates the
-      new name and sets it in the @ref KPIM::IdentityManager */
+      new name and sets it in the KPIM::IdentityManager */
   void slotRenameIdentity( QListViewItem *, const QString &, int );
   void slotContextMenu( KListView*, QListViewItem *, const QPoint & );
   void slotSetAsDefault();
   void slotIdentitySelectionChanged();
 
-protected: // methods
+private: // methods
   void refreshList();
 
-protected: // data members
+private: // data members
   KMail::IdentityDialog   * mIdentityDialog;
   int            mOldNumberOfIdentities;
 
@@ -277,37 +303,38 @@ protected: // data members
 
 //
 //
-// NetworkPage
+// AccountsPage
 //
 //
 
 // subclasses: one class per tab:
-class NetworkPageSendingTab : public ConfigModuleTab {
+class AccountsPageSendingTab : public ConfigModuleTab {
   Q_OBJECT
 public:
-  NetworkPageSendingTab( QWidget * parent=0, const char * name=0 );
+  AccountsPageSendingTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-  void load();
   void save();
-  void defaults() {}
 
 signals:
   void transportListChanged( const QStringList & );
 
-protected slots:
+private slots:
   void slotTransportSelected();
   void slotAddTransport();
   void slotModifySelectedTransport();
   void slotRemoveSelectedTransport();
-  void slotTransportUp();
-  void slotTransportDown();
+  void slotSetDefaultTransport();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   ListView    *mTransportList;
   QPushButton *mModifyTransportButton;
   QPushButton *mRemoveTransportButton;
-  QPushButton *mTransportUpButton;
-  QPushButton *mTransportDownButton;
+  QPushButton *mSetDefaultTransportButton;
   QCheckBox   *mConfirmSendCheck;
   QComboBox   *mSendOnCheckCombo;
   QComboBox   *mSendMethodCombo;
@@ -318,29 +345,31 @@ protected:
 };
 
 
-class NetworkPageReceivingTab : public ConfigModuleTab {
+class AccountsPageReceivingTab : public ConfigModuleTab {
   Q_OBJECT
 public:
-  NetworkPageReceivingTab( QWidget * parent=0, const char * name=0 );
+  AccountsPageReceivingTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-  void load();
   void save();
-  void defaults() {}
 
 signals:
   void accountListChanged( const QStringList & );
 
-protected slots:
+private slots:
   void slotAccountSelected();
   void slotAddAccount();
   void slotModifySelectedAccount();
   void slotRemoveSelectedAccount();
   void slotEditNotifications();
+  void slotTweakAccountList();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   QStringList occupiedNames();
 
-protected:
+private:
   ListView      *mAccountList;
   QPushButton   *mModifyAccountButton;
   QPushButton   *mRemoveAccountButton;
@@ -359,22 +388,22 @@ protected:
   QValueList< ModifiedAccountsType* >  mModifiedAccounts;
 };
 
-class NetworkPage : public ConfigModuleWithTabs {
+class KDE_EXPORT AccountsPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
-  NetworkPage( QWidget * parent=0, const char * name=0 );
+  AccountsPage( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
-  typedef NetworkPageSendingTab SendingTab;
-  typedef NetworkPageReceivingTab ReceivingTab;
+  typedef AccountsPageSendingTab SendingTab;
+  typedef AccountsPageReceivingTab ReceivingTab;
 
 signals:
   void transportListChanged( const QStringList & );
   void accountListChanged( const QStringList & );
 
-protected:
+private:
   SendingTab   *mSendingTab;
   ReceivingTab *mReceivingTab;
 };
@@ -391,25 +420,26 @@ class AppearancePageFontsTab : public ConfigModuleTab {
 public:
   AppearancePageFontsTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-  void load();
   void save();
-  void defaults() {}
 
   void installProfile( KConfig * profile );
 
-protected slots:
+private slots:
   void slotFontSelectorChanged( int );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   void updateFontSelector();
 
-protected:
+private:
   QCheckBox    *mCustomFontCheck;
   QComboBox    *mFontLocationCombo;
   KFontChooser *mFontChooser;
 
   int          mActiveFontIndex;
-  QFont        mFont[10];
+  QFont        mFont[14];
 };
 
 class AppearancePageColorsTab : public ConfigModuleTab {
@@ -417,13 +447,16 @@ class AppearancePageColorsTab : public ConfigModuleTab {
 public:
   AppearancePageColorsTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-  void load();
   void save();
-  void defaults() {}
 
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox    *mCustomColorCheck;
   ColorListBox *mColorList;
   QCheckBox    *mRecycleColorCheck;
@@ -435,13 +468,15 @@ public:
   AppearancePageLayoutTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
-protected: // data
-  QCheckBox    *mShowColorbarCheck;
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private: // data
   QButtonGroup *mFolderListGroup;
   QButtonGroup *mMIMETreeLocationGroup;
   QButtonGroup *mMIMETreeModeGroup;
@@ -455,15 +490,16 @@ public:
 
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
-protected: // methods
+private: // methods
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   void setDateDisplay( int id, const QString & format );
 
-protected: // data
+private: // data
   QCheckBox    *mMessageSizeCheck;
   QCheckBox    *mAttachmentCheck;
   QCheckBox    *mNestedMessagesCheck;
@@ -473,6 +509,35 @@ protected: // data
   QLineEdit    *mCustomDateFormatEdit;
 };
 
+class AppearancePageReaderTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  AppearancePageReaderTab( QWidget * parent=0, const char * name=0 );
+
+  QString helpAnchor() const;
+
+  void save();
+  void installProfile( KConfig * profile );
+
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+  void readCurrentFallbackCodec();
+  void readCurrentOverrideCodec();
+
+private: // data
+  QCheckBox *mShowColorbarCheck;
+  QCheckBox *mShowSpamStatusCheck;
+  QCheckBox *mShowEmoticonsCheck;
+  QCheckBox *mShowExpandQuotesMark;
+  KIntSpinBox  *mCollapseQuoteLevelSpin;
+  QCheckBox *mShrinkQuotesCheck;
+  QComboBox *mCharsetCombo;
+  QComboBox *mOverrideCharsetCombo;
+};
+
+
 class AppearancePageSystemTrayTab : public ConfigModuleTab {
   Q_OBJECT
 public:
@@ -480,17 +545,18 @@ public:
 
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
+private:
+  virtual void doLoadFromGlobalSettings();
+
 private: // data
   QCheckBox    *mSystemTrayCheck;
   QButtonGroup *mSystemTrayGroup;
 };
 
-class AppearancePage : public ConfigModuleWithTabs {
+class KDE_EXPORT AppearancePage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   AppearancePage( QWidget * parent=0, const char * name=0 );
@@ -502,13 +568,15 @@ public:
   typedef AppearancePageColorsTab ColorsTab;
   typedef AppearancePageLayoutTab LayoutTab;
   typedef AppearancePageHeadersTab HeadersTab;
+  typedef AppearancePageReaderTab ReaderTab;
   typedef AppearancePageSystemTrayTab SystemTrayTab;
 
-protected:
+private:
   FontsTab      *mFontsTab;
   ColorsTab     *mColorsTab;
   LayoutTab     *mLayoutTab;
   HeadersTab    *mHeadersTab;
+  ReaderTab     *mReaderTab;
   SystemTrayTab *mSystemTrayTab;
 };
 
@@ -524,15 +592,20 @@ public:
   ComposerPageGeneralTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults();
   void installProfile( KConfig * profile );
+protected slots:
+  void slotConfigureRecentAddresses();
+  void slotConfigureCompletionOrder();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+
+private:
   QCheckBox     *mAutoAppSignFileCheck;
   QCheckBox     *mSmartQuoteCheck;
   QCheckBox     *mAutoRequestMDNCheck;
+  QCheckBox	*mShowRecentAddressesInComposer;
   QCheckBox     *mWordWrapCheck;
   KIntSpinBox   *mWrapColumnSpin;
   KIntSpinBox   *mAutoSave;
@@ -546,21 +619,20 @@ public:
   ComposerPagePhrasesTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
 
-protected slots:
+private slots:
   void slotNewLanguage();
   void slotRemoveLanguage();
   void slotLanguageChanged( const QString& );
   void slotAddNewLanguage( const QString& );
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
   void setLanguageItemInformation( int index );
   void saveActiveLanguageItem();
 
-protected:
+private:
   LanguageComboBox *mPhraseLanguageCombo;
   QPushButton      *mRemoveButton;
   QLineEdit        *mPhraseReplyEdit;
@@ -572,18 +644,52 @@ protected:
   LanguageItemList mLanguageList;
 };
 
+class ComposerPageTemplatesTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  ComposerPageTemplatesTab( QWidget * parent=0, const char * name=0 );
+  QString helpAnchor() const;
+
+  void save();
+
+private slots:
+
+private:
+  virtual void doLoadFromGlobalSettings();
+
+private:
+    TemplatesConfiguration* mWidget;
+};
+
+class ComposerPageCustomTemplatesTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  ComposerPageCustomTemplatesTab( QWidget * parent=0, const char * name=0 );
+  QString helpAnchor() const;
+
+  void save();
+
+private slots:
+
+private:
+  virtual void doLoadFromGlobalSettings();
+
+private:
+    CustomTemplates* mWidget;
+};
+
 class ComposerPageSubjectTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageSubjectTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
 
+private:
+  virtual void doLoadFromGlobalSettings();
 
-protected:
+private:
   SimpleStringListEditor *mReplyListEditor;
   QCheckBox              *mReplaceReplyPrefixCheck;
   SimpleStringListEditor *mForwardListEditor;
@@ -596,14 +702,17 @@ public:
   ComposerPageCharsetTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
 
-protected slots:
+private slots:
   void slotVerifyCharset(QString&);
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   SimpleStringListEditor *mCharsetListEditor;
   QCheckBox              *mKeepReplyCharsetCheck;
 };
@@ -614,18 +723,21 @@ public:
   ComposerPageHeadersTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
 
-protected slots:
+private slots:
   void slotMimeHeaderSelectionChanged();
   void slotMimeHeaderNameChanged( const QString & );
   void slotMimeHeaderValueChanged( const QString & );
   void slotNewMimeHeader();
   void slotRemoveMimeHeader();
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox   *mCreateOwnMessageIdCheck;
   QLineEdit   *mMessageIdSuffixEdit;
   QRegExpValidator *mMessageIdSuffixValidator;
@@ -643,20 +755,22 @@ public:
   ComposerPageAttachmentsTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
 
-protected slots:
+private slots:
   void slotOutlookCompatibleClicked();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox   *mOutlookCompatibleCheck;
   QCheckBox   *mMissingAttachmentDetectionCheck;
   SimpleStringListEditor *mAttachWordsListEditor;
 };
 
-class ComposerPage : public ConfigModuleWithTabs {
+class KDE_EXPORT ComposerPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   ComposerPage( QWidget * parent=0, const char * name=0 );
@@ -666,14 +780,18 @@ public:
   // hrmpf. moc doesn't like nested classes with slots/signals...:
   typedef ComposerPageGeneralTab GeneralTab;
   typedef ComposerPagePhrasesTab PhrasesTab;
+  typedef ComposerPageTemplatesTab TemplatesTab;
+  typedef ComposerPageCustomTemplatesTab CustomTemplatesTab;
   typedef ComposerPageSubjectTab SubjectTab;
   typedef ComposerPageCharsetTab CharsetTab;
   typedef ComposerPageHeadersTab HeadersTab;
   typedef ComposerPageAttachmentsTab AttachmentsTab;
 
-protected:
+private:
   GeneralTab  *mGeneralTab;
   PhrasesTab  *mPhrasesTab;
+  TemplatesTab  *mTemplatesTab;
+  CustomTemplatesTab  *mCustomTemplatesTab;
   SubjectTab  *mSubjectTab;
   CharsetTab  *mCharsetTab;
   HeadersTab  *mHeadersTab;
@@ -692,13 +810,15 @@ public:
   SecurityPageGeneralTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
-
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox    *mExternalReferences;
   QCheckBox    *mHtmlMailCheck;
   QCheckBox    *mNoMDNsWhenEncryptedCheck;
@@ -715,12 +835,15 @@ public:
 
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   ComposerCryptoConfiguration* mWidget;
 };
 
@@ -731,49 +854,44 @@ public:
 
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
 private slots:
   void slotReenableAllWarningsClicked();
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   WarningConfiguration* mWidget;
 };
 
-class SecurityPageSMimeTab : public ConfigModuleTab {
+class SecurityPageSMimeTab : public ConfigModuleTab, public DCOPObject {
   Q_OBJECT
+  K_DCOP
 public:
   SecurityPageSMimeTab( QWidget * parent=0, const char * name=0 );
+  ~SecurityPageSMimeTab();
 
   QString helpAnchor() const;
 
-  void load();
+  // Can't use k_dcop here. dcopidl can't parse this file, dcopidlng has a namespace bug.
   void save();
-  void defaults() {}
   void installProfile( KConfig * profile );
 
-protected:
-  Kleo::CryptoConfigEntry* configEntry( const char* componentName,
-                                        const char* groupName,
-                                        const char* entryName,
-                                        int argType,
-                                        bool isList );
+private slots:
+  void slotUpdateHTTPActions();
+
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
 
 private:
   SMimeConfiguration* mWidget;
-  // Checkboxes
-  Kleo::CryptoConfigEntry* mCheckUsingOCSPConfigEntry;
-  Kleo::CryptoConfigEntry* mEnableOCSPsendingConfigEntry;
-  Kleo::CryptoConfigEntry* mDoNotCheckCertPolicyConfigEntry;
-  Kleo::CryptoConfigEntry* mNeverConsultConfigEntry;
-  Kleo::CryptoConfigEntry* mFetchMissingConfigEntry;
-  // Other widgets
-  Kleo::CryptoConfigEntry* mOCSPResponderURLConfigEntry;
-  Kleo::CryptoConfigEntry* mOCSPResponderSignature;
-
   Kleo::CryptoConfig* mConfig;
 };
 
@@ -786,18 +904,20 @@ public:
 
   QString helpAnchor() const;
 
-  void load();
   void save();
-  void defaults() {}
+
+private:
+  virtual void doLoadOther();
+  //virtual void doResetToDefaultsOther();
 
 private:
   Kleo::BackendConfigWidget * mBackendConfig;
 };
 
-class SecurityPage : public ConfigModuleWithTabs {
+class KDE_EXPORT SecurityPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
-  SecurityPage(	QWidget * parent=0, const char * name=0 );
+  SecurityPage( QWidget * parent=0, const char * name=0 );
 
   QString helpAnchor() const;
 
@@ -810,7 +930,7 @@ public:
   typedef SecurityPageSMimeTab SMimeTab;
   typedef SecurityPageCryptPlugTab CryptPlugTab;
 
-protected:
+private:
   GeneralTab    *mGeneralTab;
   ComposerCryptoTab *mComposerCryptoTab;
   WarningTab    *mWarningTab;
@@ -830,34 +950,44 @@ class MiscPageFolderTab : public ConfigModuleTab {
 public:
   MiscPageFolderTab( QWidget * parent=0, const char * name=0 );
 
-  void load();
   void save();
-  void defaults() {}
  QString helpAnchor() const;
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox    *mEmptyFolderConfirmCheck;
   QCheckBox    *mExcludeImportantFromExpiry;
   QComboBox    *mLoopOnGotoUnread;
-  QCheckBox    *mJumpToUnread;
   QComboBox    *mMailboxPrefCombo;
+  QComboBox    *mActionEnterFolder;
   QCheckBox    *mEmptyTrashCheck;
+#ifdef HAVE_INDEXLIB
+  QCheckBox    *mIndexingEnabled;
+#endif
   QCheckBox    *mDelayedMarkAsRead;
   KIntSpinBox  *mDelayedMarkTime;
   QCheckBox    *mShowPopupAfterDnD;
-  KMFolderComboBox *mOnStartupOpenFolder;
+  KMail::FolderRequester *mOnStartupOpenFolder;
 };
 
 class MiscPageGroupwareTab : public ConfigModuleTab  {
   Q_OBJECT
 public:
   MiscPageGroupwareTab( QWidget * parent=0, const char * name=0 );
-  void load();
   void save();
-  void defaults() {}
   QString helpAnchor() const;
+
 private slots:
+  void slotStorageFormatChanged( int );
   void slotLegacyBodyInvitesToggled( bool on );
+
+private:
+  virtual void doLoadFromGlobalSettings();
+
 private:
   QCheckBox* mEnableGwCB;
   QCheckBox* mEnableImapResCB;
@@ -865,8 +995,14 @@ private:
   QWidget* mBox;
   QVBox* gBox;
 
+  QComboBox* mStorageFormatCombo;
   QComboBox* mLanguageCombo;
-  KMFolderComboBox* mFolderCombo;
+
+  QLabel* mFolderComboLabel;
+  QWidgetStack* mFolderComboStack;
+  KMail::FolderRequester* mFolderCombo; // in the widgetstack
+  KMail::AccountComboBox* mAccountCombo; // in the widgetstack
+
   QCheckBox* mHideGroupwareFolders;
   QCheckBox* mAutoResCB;
   QCheckBox* mAutoDeclConflCB;
@@ -874,9 +1010,10 @@ private:
 
   QCheckBox* mLegacyMangleFromTo;
   QCheckBox* mLegacyBodyInvites;
+  QCheckBox* mAutomaticSending;
 };
 
-class MiscPage : public ConfigModuleWithTabs {
+class KDE_EXPORT MiscPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   MiscPage( QWidget * parent=0, const char * name=0 );
