@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -36,11 +36,15 @@
 #include "headerstyle.h"
 
 #include "headerstrategy.h"
+#include "kmkernel.h"
 #include "linklocator.h"
 #include "kmmessage.h"
-#include "kmkernel.h"
+#include "spamheaderanalyzer.h"
+#include "globalsettings.h"
 
-#include <libkdepim/email.h>
+#include <libemailfunctions/email.h>
+#include <libkdepim/kxface.h>
+using namespace KPIM;
 
 #include <mimelib/string.h>
 #include <mimelib/field.h>
@@ -49,12 +53,13 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <kglobal.h>
-#include "kimproxy.h"
+#include <kimproxy.h>
 #include <kabc/stdaddressbook.h>
 #include <kabc/addresseelist.h>
 #include <kmdcodec.h>
 #include <qdatetime.h>
 #include <qbuffer.h>
+#include <qbitmap.h>
 #include <qimage.h>
 #include <qapplication.h>
 #include <qregexp.h>
@@ -83,8 +88,9 @@ namespace KMail {
 #endif
 
   // ### tmp wrapper to make kmreaderwin code working:
-  static QString strToHtml( const QString & str, bool preserveBlanks=true ) {
-    return LinkLocator::convertToHtml( str, preserveBlanks );
+  static QString strToHtml( const QString & str,
+                            int flags = LinkLocator::PreserveSpaces ) {
+    return LinkLocator::convertToHtml( str, flags );
   }
 
   //
@@ -93,23 +99,23 @@ namespace KMail {
   //
 
   class BriefHeaderStyle : public HeaderStyle {
-    friend class HeaderStyle;
+    friend class ::KMail::HeaderStyle;
   protected:
     BriefHeaderStyle() : HeaderStyle() {}
     virtual ~BriefHeaderStyle() {}
-    
+
   public:
     const char * name() const { return "brief"; }
     const HeaderStyle * next() const { return plain(); }
     const HeaderStyle * prev() const { return fancy(); }
 
     QString format( const KMMessage * message, const HeaderStrategy * strategy,
-		    const QString & vCardName, bool printing ) const;
+                    const QString & vCardName, bool printing ) const;
   };
 
   QString BriefHeaderStyle::format( const KMMessage * message,
-				    const HeaderStrategy * strategy,
-				    const QString & vCardName, bool printing ) const {
+                                    const HeaderStrategy * strategy,
+                                    const QString & vCardName, bool printing ) const {
     if ( !message ) return QString::null;
     if ( !strategy )
       strategy = HeaderStrategy::brief();
@@ -146,16 +152,19 @@ namespace KMail {
 
     if ( strategy->showHeader( "subject" ) )
       headerStr += "<div dir=\"" + subjectDir + "\">\n"
-	           "<b style=\"font-size:130%\">" +
-			   strToHtml( message->subject() ) +
-			   "</b></div>\n";
+                   "<b style=\"font-size:130%\">" +
+                           strToHtml( message->subject() ) +
+                           "</b></div>\n";
 
     QStringList headerParts;
 
     if ( strategy->showHeader( "from" ) ) {
-      QString fromPart = KMMessage::emailAddrAsAnchor( message->from(), true );
+      QString fromStr = message->from();
+      if ( fromStr.isEmpty() ) // no valid email in from, maybe just a name
+        fromStr = message->fromStrip(); // let's use that
+      QString fromPart = KMMessage::emailAddrAsAnchor( fromStr, true );
       if ( !vCardName.isEmpty() )
-	fromPart += "&nbsp;&nbsp;<a href=\"" + vCardName + "\">" + i18n("[vCard]") + "</a>";
+        fromPart += "&nbsp;&nbsp;<a href=\"" + vCardName + "\">" + i18n("[vCard]") + "</a>";
       headerParts << fromPart;
     }
 
@@ -185,26 +194,26 @@ namespace KMail {
   //
 
   class PlainHeaderStyle : public HeaderStyle {
-    friend class HeaderStyle;
+    friend class ::KMail::HeaderStyle;
   protected:
     PlainHeaderStyle() : HeaderStyle() {}
     virtual ~PlainHeaderStyle() {}
-    
+
   public:
     const char * name() const { return "plain"; }
     const HeaderStyle * next() const { return fancy(); }
     const HeaderStyle * prev() const { return brief(); }
 
     QString format( const KMMessage * message, const HeaderStrategy * strategy,
-		    const QString & vCardName, bool printing ) const;
+                    const QString & vCardName, bool printing ) const;
 
   private:
     QString formatAllMessageHeaders( const KMMessage * message ) const;
   };
 
   QString PlainHeaderStyle::format( const KMMessage * message,
-				    const HeaderStrategy * strategy,
-				    const QString & vCardName, bool printing ) const {
+                                    const HeaderStrategy * strategy,
+                                    const QString & vCardName, bool printing ) const {
     if ( !message ) return QString::null;
     if ( !strategy )
       strategy = HeaderStrategy::rich();
@@ -238,44 +247,56 @@ namespace KMail {
       dateString = message->dateStr();
     }
 
-    QString headerStr = QString("<div class=\"header\" dir=\"%1\">").arg(dir);
+    QString headerStr;
 
     if ( strategy->headersToDisplay().isEmpty()
-	 && strategy->defaultPolicy() == HeaderStrategy::Display ) {
-      // crude way to emulate "all" headers:
+         && strategy->defaultPolicy() == HeaderStrategy::Display ) {
+      // crude way to emulate "all" headers - Note: no strings have
+      // i18n(), so direction should always be ltr.
+      headerStr= QString("<div class=\"header\" dir=\"ltr\">");
       headerStr += formatAllMessageHeaders( message );
       return headerStr + "</div>";
     }
 
+    headerStr = QString("<div class=\"header\" dir=\"%1\">").arg(dir);
+
     //case HdrLong:
     if ( strategy->showHeader( "subject" ) )
       headerStr += QString("<div dir=\"%1\"><b style=\"font-size:130%\">" +
-			   strToHtml(message->subject()) + "</b></div>\n")
+                           strToHtml(message->subject()) + "</b></div>\n")
                         .arg(subjectDir);
 
     if ( strategy->showHeader( "date" ) )
       headerStr.append(i18n("Date: ") + strToHtml(dateString)+"<br>\n");
-    
+
+#if 0
     // Get Instant Messaging presence
     QString presence;
     QString kabcUid;
     if ( strategy->showHeader( "status" ) )
     {
-      KABC::AddressBook *addressBook = KABC::StdAddressBook::self();
-      KABC::AddresseeList addresses = addressBook->findByEmail( KPIM::getEmailAddr( message->from() ) );
+      KABC::AddressBook *addressBook = KABC::StdAddressBook::self( true );
+      KABC::AddresseeList addresses = addressBook->findByEmail( KPIM::getFirstEmailAddress( message->from() ) );
       ::KIMProxy *imProxy = KMKernel::self()->imProxy();
       kabcUid = addresses[0].uid();
       presence = imProxy->presenceString( kabcUid );
     }
- 
+#endif
+
     if ( strategy->showHeader( "from" ) ) {
+      QString fromStr = message->from();
+      if ( fromStr.isEmpty() ) // no valid email in from, maybe just a name
+        fromStr = message->fromStrip(); // let's use that
       headerStr.append(i18n("From: ") +
-		       KMMessage::emailAddrAsAnchor(message->from(),FALSE));
+          KMMessage::emailAddrAsAnchor( fromStr, false) );
       if ( !vCardName.isEmpty() )
         headerStr.append("&nbsp;&nbsp;<a href=\"" + vCardName +
               "\">" + i18n("[vCard]") + "</a>" );
+#if 0
       if ( !presence.isEmpty() && strategy->showHeader( "status" ) )
         headerStr.append("&nbsp;&nbsp;(<span name=\"presence-" + kabcUid + "\">" + presence + "</span>)" );
+#endif
+
       if ( strategy->showHeader( "organization" )
           && !message->headerField("Organization").isEmpty())
         headerStr.append("&nbsp;&nbsp;(" +
@@ -285,7 +306,7 @@ namespace KMail {
 
     if ( strategy->showHeader( "to" ) )
       headerStr.append(i18n("To: ")+
-		       KMMessage::emailAddrAsAnchor(message->to(),FALSE) + "<br>\n");
+                       KMMessage::emailAddrAsAnchor(message->to(),FALSE) + "<br>\n");
 
     if ( strategy->showHeader( "cc" ) && !message->cc().isEmpty() )
       headerStr.append(i18n("CC: ")+
@@ -323,29 +344,82 @@ namespace KMail {
   //
 
   class FancyHeaderStyle : public HeaderStyle {
-    friend class HeaderStyle;
+    friend class ::KMail::HeaderStyle;
   protected:
     FancyHeaderStyle() : HeaderStyle() {}
     virtual ~FancyHeaderStyle() {}
-    
+
   public:
     const char * name() const { return "fancy"; }
     const HeaderStyle * next() const { return brief(); }
     const HeaderStyle * prev() const { return plain(); }
 
     QString format( const KMMessage * message, const HeaderStrategy * strategy,
-            const QString & vCardName, bool printing ) const;
-    static QString imgToDataUrl( const QImage &image );
+                    const QString & vCardName, bool printing ) const;
+    static QString imgToDataUrl( const QImage & image,
+                                 const char *fmt = "PNG" );
+
+  private:
+    static QString drawSpamMeter( double percent, const QString & filterHeader );
 
   };
 
+  QString FancyHeaderStyle::drawSpamMeter( double percent,
+                                           const QString & filterHeader )
+  {
+    QImage meterBar( 20, 1, 8, 24 );
+    const unsigned short gradient[20][3] = {
+      {   0, 255,   0 },
+      {  27, 254,   0 },
+      {  54, 252,   0 },
+      {  80, 250,   0 },
+      { 107, 249,   0 },
+      { 135, 247,   0 },
+      { 161, 246,   0 },
+      { 187, 244,   0 },
+      { 214, 242,   0 },
+      { 241, 241,   0 },
+      { 255, 228,   0 },
+      { 255, 202,   0 },
+      { 255, 177,   0 },
+      { 255, 151,   0 },
+      { 255, 126,   0 },
+      { 255, 101,   0 },
+      { 255,  76,   0 },
+      { 255,  51,   0 },
+      { 255,  25,   0 },
+      { 255,   0,   0 }
+    };
+    meterBar.setColor( 21, qRgb( 255, 255, 255 ) );
+    meterBar.setColor( 22, qRgb( 170, 170, 170 ) );
+    if ( percent < 0 ) // grey is for errors
+      meterBar.fill( 22 );
+    else {
+      meterBar.fill( 21 );
+      int max = QMIN( 20, static_cast<int>( percent ) / 5 );
+      for ( int i = 0; i < max; ++i ) {
+        meterBar.setColor( i+1, qRgb( gradient[i][0], gradient[i][1],
+                                      gradient[i][2] ) );
+        meterBar.setPixel( i, 0, i+1 );
+      }
+    }
+    QString titleText = i18n("%1% probability of being spam.\n\nFull report:\n%2")
+                     .arg( QString::number( percent ), filterHeader );
+    return QString("<img src=\"%1\" width=\"%2\" height=\"%3\" style=\"border: 1px solid black;\" title=\"%4\"> &nbsp;")
+      .arg( imgToDataUrl( meterBar, "PPM" ), QString::number( 20 ),
+            QString::number( 5 ), titleText );
+  }
+
+
   QString FancyHeaderStyle::format( const KMMessage * message,
-				    const HeaderStrategy * strategy,
-				    const QString & vCardName, bool printing ) const {
+                                    const HeaderStrategy * strategy,
+                                    const QString & vCardName, bool printing ) const {
     if ( !message ) return QString::null;
     if ( !strategy )
       strategy = HeaderStrategy::rich();
 
+    KConfigGroup configReader( KMKernel::config(), "Reader" );
+
     // ### from kmreaderwin begin
     // The direction of the header is determined according to the direction
     // of the application layout.
@@ -377,116 +451,211 @@ namespace KMail {
       dateString = message->dateStr();
     }
 
+    // Spam header display.
+    // If the spamSpamStatus config value is true then we look for headers
+    // from a few spam filters and try to create visually meaningful graphics
+    // out of the spam scores.
+
+    QString spamHTML;
+
+    if ( configReader.readBoolEntry( "showSpamStatus", true ) ) {
+      SpamScores scores = SpamHeaderAnalyzer::getSpamScores( message );
+      for ( SpamScoresIterator it = scores.begin(); it != scores.end(); ++it )
+        spamHTML += (*it).agent() + " " +
+                    drawSpamMeter( (*it).score(), (*it).spamHeader() );
+    }
+
     QString userHTML;
     QString presence;
-    
+
     // IM presence and kabc photo
-    // Check first that KIMProxy has any IM presence data, to save hitting KABC 
-    // unless really necessary
+
     ::KIMProxy *imProxy = KMKernel::self()->imProxy();
     QString kabcUid;
-    if ( ( strategy->showHeader( "status" ) || strategy->showHeader( "statuspic" ) ) )
+    KABC::AddressBook *addressBook = KABC::StdAddressBook::self( true );
+    KABC::AddresseeList addresses = addressBook->findByEmail( KPIM::getFirstEmailAddress( message->from() ) );
+
+    QString photoURL;
+    int photoWidth = 60;
+    int photoHeight = 60;
+    if( addresses.count() == 1 )
     {
-      if ( imProxy->initialize() )
-      {
-        KABC::AddressBook *addressBook = KABC::StdAddressBook::self();
-        KABC::AddresseeList addresses = addressBook->findByEmail( KPIM::getEmailAddr( message->from() ) );
-    
-        if( addresses.count() == 1 )
-        {
-          // kabcUid is embedded in im: URIs to indicate which IM contact to message
-          kabcUid = addresses[0].uid();
-          
+      // kabcUid is embedded in im: URIs to indicate which IM contact to message
+      kabcUid = addresses[0].uid();
+
+      if ( imProxy->initialize() ) {
           // im status
           presence = imProxy->presenceString( kabcUid );
           if ( !presence.isEmpty() )
-          {  
+          {
             QString presenceIcon = QString::fromLatin1( " <img src=\"%1\"/>" )
                 .arg( imgToDataUrl( imProxy->presenceIcon( kabcUid ).convertToImage() ) );
             presence += presenceIcon;
           }
-          // picture
-          if ( strategy->showHeader( "statuspic" ) )
-          {
-            QString photoURL;
-            if ( addresses[0].photo().isIntern() )
-            {
-              // get photo data and convert to data: url
-              //kdDebug( 5006 ) << "INTERNAL photo found" << endl;
-              QImage photo = addresses[0].photo().data();
-              if ( !photo.isNull() )
-              {
-                photoURL = imgToDataUrl( photo );
-              }
-            }
-            else
-            {
-              //kdDebug( 5006 ) << "URL found" << endl;
-              photoURL = addresses[0].photo().url();
-              if ( photoURL.startsWith("/") )
-                photoURL.prepend( "file:" );
-            }
-            if( !photoURL.isEmpty() )
-            {
-              //kdDebug( 5006 ) << "Got a photo: " << photoURL << endl;
-              userHTML = QString("<img src=\"%1\" width=\"60\" height=\"60\">").arg( photoURL );
-              if ( presence.isEmpty() )
-              {  
-                userHTML = QString("<div class=\"senderpic\">") + userHTML + "</div>";
-              }
-              else
-                userHTML = QString( "<div class=\"senderpic\">"
-                                      "<a href=\"im:%1\">%2<div class=\"senderstatus\"><span name=\"presence-%2\">%3</span></div></a>"
-                                      "</div>" ).arg( kabcUid )
-                                                .arg( userHTML )
-                                                .arg( presence );
-            }
-          }
-        }
-        else
+      }
+      // picture
+      if ( addresses[0].photo().isIntern() )
+      {
+        // get photo data and convert to data: url
+        //kdDebug( 5006 ) << "INTERNAL photo found" << endl;
+        QImage photo = addresses[0].photo().data();
+        if ( !photo.isNull() )
         {
-          kdDebug( 5006 ) << "Multiple / No addressees matched email address; Count is " << addresses.count() << endl;
-          userHTML = "&nbsp;";
+          photoWidth = photo.width();
+          photoHeight = photo.height();
+          // scale below 60, otherwise it can get way too large
+          if ( photoHeight > 60 ) {
+            double ratio = ( double )photoHeight / ( double )photoWidth;
+            photoHeight = 60;
+            photoWidth = (int)( 60 / ratio );
+            photo = photo.smoothScale( photoWidth, photoHeight );
+          }
+          photoURL = imgToDataUrl( photo );
         }
       }
-// Disabled 'Launch IM' link in headers - Will
-//      else
-//        if ( imProxy->imAppsAvailable() )
-//          presence = "<a name=\"launchim\" href=\"kmail:startIMApp\">" + i18n("Launch IM") + "</a></span>";
+      else
+      {
+        //kdDebug( 5006 ) << "URL found" << endl;
+        photoURL = addresses[0].photo().url();
+        if ( photoURL.startsWith("/") )
+          photoURL.prepend( "file:" );
+      }
     }
+    else // TODO: find a usable one
+    {
+      kdDebug( 5006 ) << "Multiple / No addressees matched email address; Count is " << addresses.count() << endl;
+      userHTML = "&nbsp;";
+    }
+
+    if( photoURL.isEmpty() ) {
+      // no photo, look for a Face header
+      QString faceheader = message->headerField( "Face" );
+      if ( !faceheader.isEmpty() ) {
+        QImage faceimage;
+
+        kdDebug( 5006 ) << "Found Face: header" << endl;
+
+        QCString facestring = faceheader.utf8();
+        // Spec says header should be less than 998 bytes
+        // Face: is 5 characters
+        if ( facestring.length() < 993 ) {
+          QByteArray facearray;
+          KCodecs::base64Decode(facestring, facearray);
+    
+          QImage faceimage;
+          if ( faceimage.loadFromData( facearray, "png" ) ) {
+            // Spec says image must be 48x48 pixels
+            if ( (48 == faceimage.width()) && (48 == faceimage.height()) ) {
+              photoURL = imgToDataUrl( faceimage );
+              photoWidth = 48;
+              photoHeight = 48;
+            } else {
+              kdDebug( 5006 ) << "Face: header image is" << faceimage.width() << "by" << faceimage.height() <<"not 48x48 Pixels" << endl;
+            }
+          } else {
+            kdDebug( 5006 ) << "Failed to load decoded png from Face: header" << endl;
+          }
+        } else {
+          kdDebug( 5006 ) << "Face: header too long at " << facestring.length() << endl;
+        }
+      }
+    }
+
+    if( photoURL.isEmpty() )
+    {
+      // no photo, look for a X-Face header
+      QString xfaceURL;
+      QString xfhead = message->headerField( "X-Face" );
+      if ( !xfhead.isEmpty() )
+      {
+        KXFace xf;
+        photoURL = imgToDataUrl( xf.toImage( xfhead ) );
+        photoWidth = 48;
+        photoHeight = 48;
+
+      }
+    }
+
+    if( !photoURL.isEmpty() )
+    {
+        //kdDebug( 5006 ) << "Got a photo: " << photoURL << endl;
+      userHTML = QString("<img src=\"%1\" width=\"%2\" height=\"%3\">")
+          .arg( photoURL ).arg( photoWidth ).arg( photoHeight );
+      if ( presence.isEmpty() ) {
+        userHTML = QString("<div class=\"senderpic\">") + userHTML + "</div>";
+      } else {
+        userHTML = QString( "<div class=\"senderpic\">"
+            "<a href=\"im:%1\">%2<div class=\"senderstatus\">"
+            "<span name=\"presence-%3\">%4</span></div></a>"
+            "</div>" ).arg( kabcUid )
+            .arg( userHTML )
+             .arg( kabcUid )
+            .arg( presence );
+      }
+    } else {
+       // we don't have a photo, just show presence, if we have it
+      if ( !presence.isEmpty() )
+        userHTML = QString( "<a href=\"im:%1\"><div class=\"senderstatus\">"
+            "<span name=\"presence-%2\">%3</span></div></a>" )
+            .arg( kabcUid )
+            .arg( kabcUid )
+            .arg( presence );
+    }
+#if 0
+    // Disabled 'Launch IM' link in headers - Will
+    if ( imProxy->imAppsAvailable() )
+      presence = "<a name=\"launchim\" href=\"kmail:startIMApp\">" + i18n("Launch IM") + "</a></span>";
     // do nothing - no im apps available, leave presence empty
     //presence = i18n( "DCOP/InstantMessenger not installed" );
     kdDebug( 5006 ) << "final presence: '" << presence << "'" << endl;
+#endif
+
+
     //case HdrFancy:
     // the subject line and box below for details
-    if ( strategy->showHeader( "subject" ) )
+    if ( strategy->showHeader( "subject" ) ) {
+      const int flags = LinkLocator::PreserveSpaces |
+                        ( GlobalSettings::self()->showEmoticons() ?
+                          LinkLocator::ReplaceSmileys : 0 );
       headerStr += QString("<div dir=\"%1\">%2</div>\n")
                         .arg(subjectDir)
-		        .arg(message->subject().isEmpty()?
-			     i18n("No Subject") :
-			     strToHtml(message->subject()));
+                        .arg(message->subject().isEmpty()?
+                             i18n("No Subject") :
+                             strToHtml( message->subject(), flags ));
+    }
     headerStr += "<table class=\"outer\"><tr><td width=\"100%\"><table>\n";
     //headerStr += "<table>\n";
     // from line
     // the mailto: URLs can contain %3 etc., therefore usage of multiple
     // QString::arg is not possible
-    if ( strategy->showHeader( "from" ) )
+    if ( strategy->showHeader( "from" ) ) {
+      QString fromStr = message->from();
+      if ( fromStr.isEmpty() ) // no valid email in from, maybe just a name
+        fromStr = message->fromStrip(); // let's use that
       headerStr += QString("<tr><th>%1</th>\n"
                            "<td>")
                            .arg(i18n("From: "))
-                 + KMMessage::emailAddrAsAnchor(message->from(),FALSE)
+                 + KMMessage::emailAddrAsAnchor( fromStr, false )
+                 + ( !message->headerField( "Resent-From" ).isEmpty() ? "&nbsp;"
+                                + i18n("(resent from %1)")
+                                  .arg( KMMessage::emailAddrAsAnchor(
+                                    message->headerField( "Resent-From" ),FALSE) )
+                              : QString("") )
                  + ( !vCardName.isEmpty() ? "&nbsp;&nbsp;<a href=\"" + vCardName + "\">"
                                 + i18n("[vCard]") + "</a>"
                               : QString("") )
-                 + ( ( !presence.isEmpty() && strategy->showHeader( "status" ) ) 
+#if 0
+                 + ( ( !presence.isEmpty() )
                               ? "&nbsp;&nbsp;(<span name=\"presence-" + kabcUid + "\">" + presence + "</span>)"
                               : QString("") )
+#endif
                  + ( message->headerField("Organization").isEmpty()
                               ? QString("")
                               : "&nbsp;&nbsp;("
                                 + strToHtml(message->headerField("Organization"))
                                 + ")")
                  + "</td></tr>\n";
+    }
     // to line
     if ( strategy->showHeader( "to" ) )
       headerStr.append(QString("<tr><th>%1</th>\n"
@@ -524,21 +693,26 @@ namespace KMail {
                                     .arg(onlineStatus));
     */
     headerStr.append(
-          QString("</table></td><td align=\"center\">%1</td></tr></table>\n").arg(userHTML)
-                     );
-    
+          QString( "</table></td><td align=\"center\">%1</td></tr></table>\n" ).arg(userHTML) );
+
+    if ( !spamHTML.isEmpty() )
+      headerStr.append( QString( "<div class=\"spamheader\" dir=\"%1\"><b>%2</b>&nbsp;<span style=\"padding-left: 20px;\">%3</span></div>\n")
+                        .arg( subjectDir, i18n("Spam Status:"), spamHTML ) );
+
     headerStr += "</div>\n\n";
     return headerStr;
   }
 
-QString FancyHeaderStyle::imgToDataUrl( const QImage &image )
-{
-  QByteArray ba;
-  QBuffer buffer( ba );
-  buffer.open( IO_WriteOnly );
-  image.save( &buffer, "PNG" );
-  return QString::fromLatin1("data:image/png;base64,%1").arg( KCodecs::base64Encode( ba ) );
-}  
+  QString FancyHeaderStyle::imgToDataUrl( const QImage &image, const char* fmt  )
+  {
+    QByteArray ba;
+    QBuffer buffer( ba );
+    buffer.open( IO_WriteOnly );
+    image.save( &buffer, fmt );
+    return QString::fromLatin1("data:image/%1;base64,%2")
+           .arg( fmt, KCodecs::base64Encode( ba ) );
+  }
+
   //
   // HeaderStyle abstract base:
   //
@@ -558,7 +732,7 @@ QString FancyHeaderStyle::imgToDataUrl( const QImage &image )
     case Fancy:   return fancy();
     }
     kdFatal( 5006 ) << "HeaderStyle::create(): Unknown header style ( type == "
-		    << (int)type << " ) requested!" << endl;
+                    << (int)type << " ) requested!" << endl;
     return 0; // make compiler happy
   }
 
