@@ -202,10 +202,8 @@ namespace KMail {
     namespaceDelim entries = config.entryMap( config.group() );
     namespaceDelim namespaceToDelimiter;
     for ( namespaceDelim::ConstIterator it = entries.begin(); 
-          it != entries.end(); ++it )
-    {
-      if ( it.key().startsWith( "Namespace:" ) )
-      {
+          it != entries.end(); ++it ) {
+      if ( it.key().startsWith( "Namespace:" ) ) {
         QString key = it.key().right( it.key().length() - 10 );
         namespaceToDelimiter[key] = it.data();
       }
@@ -226,18 +224,15 @@ namespace KMail {
     config.writeEntry( "loadondemand", loadOnDemand() );
     config.writeEntry( "listOnlyOpenFolders", listOnlyOpenFolders() );
     QString data;
-    for ( nsMap::Iterator it = mNamespaces.begin(); it != mNamespaces.end(); ++it )
-    {
-      if ( !it.data().isEmpty() )
-      {
+    for ( nsMap::Iterator it = mNamespaces.begin(); it != mNamespaces.end(); ++it ) {
+      if ( !it.data().isEmpty() ) {
         data = "\"" + it.data().join("\",\"") + "\"";
         config.writeEntry( QString::number( it.key() ), data );
       }
     }
     QString key;
     for ( namespaceDelim::ConstIterator it = mNamespaceToDelimiter.begin(); 
-          it != mNamespaceToDelimiter.end(); ++it )
-    {
+          it != mNamespaceToDelimiter.end(); ++it ) {
       key = "Namespace:" + it.key();
       config.writeEntry( key, it.data() );
     }
@@ -276,16 +271,17 @@ namespace KMail {
       // "keep password" checkbox. Then, we set [Passwords]Keep to
       // storePasswd(), so that the checkbox in the dialog will be
       // init'ed correctly:
-      bool store = true;
       KConfigGroup passwords( KGlobal::config(), "Passwords" );
       passwords.writeEntry( "Keep", storePasswd() );
       QString msg = i18n("You need to supply a username and a password to "
 			 "access this mailbox.");
       mPasswordDialogIsActive = true;
-      if ( PasswordDialog::getNameAndPassword( log, pass, &store, msg, false,
-					       QString::null, name(),
-					       i18n("Account:") )
-          != QDialog::Accepted ) {
+      
+      PasswordDialog dlg( msg, log, true /* store pw */, true, KMKernel::self()->mainWin() );
+      dlg.setPlainCaption( i18n("Authorization Dialog") );
+      dlg.addCommentLine( i18n("Account:"), name() );
+      int ret = dlg.exec();
+      if (ret != QDialog::Accepted ) {
         mPasswordDialogIsActive = false;
         mAskAgain = false;
         emit connectionResult( KIO::ERR_USER_CANCELED, QString::null );
@@ -294,8 +290,8 @@ namespace KMail {
       mPasswordDialogIsActive = false;
       // The user has been given the chance to change login and
       // password, so copy both from the dialog:
-      setPasswd( pass, store );
-      setLogin( log );
+      setPasswd( dlg.password(), dlg.keepPassword() );
+      setLogin( dlg.username() );
       mAskAgain = false;
     }
     // already waiting for a connection?
@@ -688,14 +684,14 @@ namespace KMail {
         QString msg = i18n("KMail has detected a prefix entry in the "
             "configuration of the account \"%1\" which is obsolete with the "
             "support of IMAP namespaces.").arg( name() );
-        if ( list.contains( QString::null ) ) {
+        if ( list.contains( "" ) ) {
           // replace empty entry with the old prefix
-          list.remove( QString::null );
+          list.remove( "" );
           list += mOldPrefix;
           mNamespaces[PersonalNS] = list;
-          if ( mNamespaceToDelimiter.contains( QString::null ) ) {
-            QString delim = mNamespaceToDelimiter[QString::null];
-            mNamespaceToDelimiter.remove( QString::null );
+          if ( mNamespaceToDelimiter.contains( "" ) ) {
+            QString delim = mNamespaceToDelimiter[""];
+            mNamespaceToDelimiter.remove( "" );
             mNamespaceToDelimiter[mOldPrefix] = delim;
           }
           kdDebug(5006) << "migratePrefix - replaced empty with " << mOldPrefix << endl;
@@ -726,7 +722,7 @@ namespace KMail {
     {
       kdDebug(5006) << "migratePrefix - no migration needed" << endl;
     }
-    mOldPrefix = QString::null;
+    mOldPrefix = "";
   }
 
   //-----------------------------------------------------------------------------
@@ -771,8 +767,7 @@ namespace KMail {
     // then try if the prefix is part of a namespace
     // exclude empty namespace
     for ( namespaceDelim::ConstIterator it = mNamespaceToDelimiter.begin(); 
-          it != mNamespaceToDelimiter.end(); ++it )
-    {
+          it != mNamespaceToDelimiter.end(); ++it ) {
       // the namespace definition sometimes contains the delimiter
       // make sure we also match this version
       QString stripped = it.key().left( it.key().length() - 1 );
@@ -782,8 +777,9 @@ namespace KMail {
       }
     }
     // see if we have an empty namespace
-    if ( mNamespaceToDelimiter.contains( QString::null ) ) {
-      return mNamespaceToDelimiter[QString::null];
+    // this should always be the case
+    if ( mNamespaceToDelimiter.contains( "" ) ) {
+      return mNamespaceToDelimiter[""];
     }
     // well, we tried
     kdDebug(5006) << "delimiterForNamespace - not found" << endl;
@@ -980,7 +976,8 @@ namespace KMail {
 
     QValueList<QGuardedPtr<KMFolder> > mSaveList = mMailCheckFolders;
     mMailCheckFolders = mFoldersQueuedForChecking;
-    kmkernel->acctMgr()->singleCheckMail(this, true);
+    if ( kmkernel->acctMgr() )
+      kmkernel->acctMgr()->singleCheckMail(this, true);
     mMailCheckFolders = mSaveList;
     mFoldersQueuedForChecking.clear();
   }
@@ -1102,10 +1099,10 @@ namespace KMail {
       {
         DwBodyPart* newparent = dwpart;
         const DwMessage* newmsg = dwmsg;
-        if ( part->originalContentTypeStr() == "MESSAGE/RFC822" &&
+        if ( part->originalContentTypeStr() == "MESSAGE/RFC822" && dwpart &&
              dwpart->Body().Message() )
         {
-          // set the encapsulated message as new parent message
+          // set the encapsulated message as the new message
           newparent = 0;
           newmsg = dwpart->Body().Message();
         }
@@ -1148,14 +1145,16 @@ namespace KMail {
      ImapAccountBase::JobIterator it = findJob(job);
      if ( it == jobsEnd() ) return;
      int errorCode = job->error();
+     KMFolder * const parent = (*it).parent;
+     const QString path = (*it).path;
      if (errorCode && errorCode != KIO::ERR_CANNOT_OPEN_FOR_WRITING)
      {
        bool cont = handleJobError( job, i18n( "Error while uploading status of messages to server: " ) + '\n' );
-       emit imapStatusChanged( (*it).parent, (*it).path, cont );
+       emit imapStatusChanged( parent, path, cont );
      }
      else
      {
-       emit imapStatusChanged( (*it).parent, (*it).path, true );
+       emit imapStatusChanged( parent, path, true );
        removeJob(it);
      }
   }
@@ -1194,7 +1193,7 @@ namespace KMail {
     {
       mListDirProgressItem = ProgressManager::createProgressItem(
           "ListDir" + name(),
-          name(),
+          QStyleSheet::escape( name() ),
           i18n("retrieving folders"),
           true,
           useSSL() || useTLS() );
@@ -1273,6 +1272,7 @@ namespace KMail {
   QString ImapAccountBase::createImapPath( const QString& parent, 
                                            const QString& folderName )
   {
+    kdDebug(5006) << "createImapPath parent="<<parent<<", folderName="<<folderName<<endl;  
     QString newName = parent;
     // strip / at the end
     if ( newName.endsWith("/") ) {
@@ -1284,7 +1284,8 @@ namespace KMail {
     if ( delim.isEmpty() ) {
       delim = "/";
     }
-    if ( !newName.endsWith( delim ) && !folderName.startsWith( delim ) ) {
+    if ( !newName.isEmpty() && 
+         !newName.endsWith( delim ) && !folderName.startsWith( delim ) ) {
       newName = newName + delim;
     }
     newName = newName + folderName;
