@@ -1,18 +1,36 @@
-// -*- c-basic-offset: 2 -*-
-// kmfoldermbox.cpp
-// Author: Stefan Taferner <taferner@alpin.or.at>
-
+/* -*- c-basic-offset: 2 -*-
+ * kmail: KDE mail client
+ * Copyright (c) 1996-1998 Stefan Taferner <taferner@kde.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ */
 #include <config.h>
 #include <qfileinfo.h>
 #include <qregexp.h>
 
 #include "kmfoldermbox.h"
-#include "kmfoldermgr.h"
+#include "folderstorage.h"
 #include "kmfolder.h"
+#include "kmkernel.h"
+#include "kmmsgdict.h"
 #include "undostack.h"
 #include "kcursorsaver.h"
 #include "jobscheduler.h"
 #include "compactionjob.h"
+#include "util.h"
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -21,6 +39,7 @@
 #include <kprocess.h>
 #include <kconfig.h>
 
+#include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
@@ -185,13 +204,11 @@ int KMFolderMbox::canAccess()
 }
 
 //-----------------------------------------------------------------------------
-int KMFolderMbox::create(bool imap)
+int KMFolderMbox::create()
 {
   int rc;
   int old_umask;
 
-  Q_UNUSED(imap);
-
   assert(!folder()->name().isEmpty());
   assert(mOpenCount == 0);
 
@@ -536,6 +553,7 @@ int KMFolderMbox::createIndexFromContents()
   QCString subjStr, dateStr, fromStr, toStr, xmarkStr, *lastStr=0;
   QCString replyToIdStr, replyToAuxIdStr, referencesStr, msgIdStr;
   QCString sizeServerStr, uidStr;
+  QCString contentTypeStr, charset;
   bool atEof = false;
   bool inHeader = true;
   KMMsgInfo* mi;
@@ -636,6 +654,29 @@ int KMFolderMbox::createIndexFromContents()
               replyToAuxIdStr.truncate( rightAngle + 1 );
           }
 
+          contentTypeStr = contentTypeStr.stripWhiteSpace();
+          charset = "";
+          if ( !contentTypeStr.isEmpty() )
+          {
+            int cidx = contentTypeStr.find( "charset=" );
+            if ( cidx != -1 ) {
+              charset = contentTypeStr.mid( cidx + 8 );
+              if ( charset[0] == '"' ) {
+                charset = charset.mid( 1 );
+              }
+              cidx = 0;
+              while ( (unsigned int) cidx < charset.length() ) {
+                if ( charset[cidx] == '"' || ( !isalnum(charset[cidx]) &&
+                    charset[cidx] != '-' && charset[cidx] != '_' ) )
+                  break;
+                ++cidx;
+              }
+              charset.truncate( cidx );
+              // kdDebug() << "KMFolderMaildir::readFileHeaderIntern() charset found: " <<
+              //              charset << " from " << contentTypeStr << endl;
+            }
+          }
+
           mi = new KMMsgInfo(folder());
           mi->init( subjStr.stripWhiteSpace(),
                     fromStr.stripWhiteSpace(),
@@ -644,11 +685,11 @@ int KMFolderMbox::createIndexFromContents()
                     xmarkStr.stripWhiteSpace(),
                     replyToIdStr, replyToAuxIdStr, msgIdStr,
                     KMMsgEncryptionStateUnknown, KMMsgSignatureStateUnknown,
-                    KMMsgMDNStateUnknown, offs, size, sizeServer, uid );
+                    KMMsgMDNStateUnknown, charset, offs, size, sizeServer, uid );
           mi->setStatus(status, xstatus);
           mi->setDate( dateStr.stripWhiteSpace() );
           mi->setDirty(false);
-          mMsgList.append(mi);
+          mMsgList.append(mi, mExportsSernums );
 
           *status = '\0';
           *xstatus = '\0';
@@ -750,6 +791,11 @@ int KMFolderMbox::createIndexFromContents()
       uid = uidStr.toULong();
       lastStr = &uidStr;
     }
+    else if (strncasecmp(line, "Content-Type:", 13) == 0)
+    {
+      contentTypeStr = QCString(line+13);
+      lastStr = &contentTypeStr;
+    }
   }
 
   if (mAutoCreateIndex)
@@ -767,8 +813,7 @@ int KMFolderMbox::createIndexFromContents()
     "most-likely not created by KMail;\nplease remove them from there if you "
     "do not want KMail to send them."));
 
-  if ( folder()->parent() )
-      folder()->parent()->manager()->invalidateFolder( kmkernel->msgDict(), folder() );
+  invalidateFolder();
   return 0;
 }
 
@@ -781,10 +826,9 @@ KMMessage* KMFolderMbox::readMsg(int idx)
   assert(mi!=0 && !mi->isMessage());
   assert(mStream != 0);
 
-  KMMessage* msg = new KMMessage(*mi); // note that mi is deleted by the line below
+  KMMessage *msg = new KMMessage(*mi); // note that mi is deleted by the line below
   mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
-  msg->fromDwString( getDwString( idx ) );
-
+  msg->fromDwString(getDwString(idx));
   return msg;
 }
 
@@ -881,7 +925,7 @@ QCString& KMFolderMbox::getMsgString(int idx, QCString &mDest)
   mDest[msgSize] = '\0';
 
   size_t newMsgSize = unescapeFrom( mDest.data(), msgSize );
-  newMsgSize = crlf2lf( mDest.data(), newMsgSize );
+  newMsgSize = KMail::Util::crlf2lf( mDest.data(), newMsgSize );
 
   return mDest;
 }
@@ -903,7 +947,7 @@ DwString KMFolderMbox::getDwString(int idx)
   msgText[msgSize] = '\0';
 
   size_t newMsgSize = unescapeFrom( msgText, msgSize );
-  newMsgSize = crlf2lf( msgText, newMsgSize );
+  newMsgSize = KMail::Util::crlf2lf( msgText, newMsgSize );
 
   DwString msgStr;
   // the DwString takes possession of msgText, so we must not delete msgText
@@ -924,13 +968,6 @@ int KMFolderMbox::addMsg( KMMessage* aMsg, int* aIndex_ret )
   bool editing = false;
   int growth = 0;
 
-/* Then we can also disable it completely, this wastes time, at least for IMAP
-  if (KMFolder::IndexOk != indexStatus()) {
-      kdDebug(5006) << "Critical error: " << location() <<
-          " has been modified by an external application while KMail was running." << endl;
-      //      exit(1); backed out due to broken nfs
-  } */
-
   if (!mStream)
   {
     opened = true;
@@ -1064,13 +1101,19 @@ if( fileD1.open( IO_WriteOnly ) ) {
   }
   ++mTotalMsgs;
 
+  if ( aMsg->attachmentState() == KMMsgAttachmentUnknown &&
+       aMsg->readyToShow() )
+    aMsg->updateAttachmentState();
+
   // store information about the position in the folder file in the message
   aMsg->setParent(folder());
   aMsg->setFolderOffset(offs);
   aMsg->setMsgSize(size);
-  idx = mMsgList.append(&aMsg->toMsgBase());
-  if (aMsg->getMsgSerNum() <= 0)
+  idx = mMsgList.append(&aMsg->toMsgBase(), mExportsSernums );
+  if ( aMsg->getMsgSerNum() <= 0 )
     aMsg->setMsgSerNum();
+  else
+    replaceMsgSerNum( aMsg->getMsgSerNum(), &aMsg->toMsgBase(), idx );
 
   // change the length of the previous message to encompass white space added
   if ((idx > 0) && (growth > 0)) {
@@ -1099,7 +1142,8 @@ if( fileD1.open( IO_WriteOnly ) ) {
     fflush(mIndexStream);
     error = ferror(mIndexStream);
 
-    error |= appendtoMsgDict(idx);
+    if ( mExportsSernums )
+      error |= appendToFolderIdsFile( idx );
 
     if (error) {
       kdWarning(5006) << "Error: Could not add message to folder (No space left on device?)" << endl;
