@@ -2,6 +2,7 @@
 
 #include <config.h>
 
+#include "globalsettings.h"
 #include "kmmsgbase.h"
 
 #include "kmfolderindex.h"
@@ -14,6 +15,7 @@ using KMail::MessageProperty;
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kcharsets.h>
+#include <kasciistringtools.h>
 #include <kmdcodec.h>
 #include <krfcdate.h>
 
@@ -123,7 +125,7 @@ void KMMsgBase::toggleStatus(const KMMsgStatus aStatus, int idx)
 {
   mDirty = true;
   KMMsgStatus oldStatus = status();
-    if ( status() & aStatus ) {
+  if ( status() & aStatus ) {
     mStatus &= ~aStatus;
   } else {
     mStatus |= aStatus;
@@ -131,14 +133,8 @@ void KMMsgBase::toggleStatus(const KMMsgStatus aStatus, int idx)
     // That is an arbitrary restriction on my part. HAR HAR HAR :) -till
     if (aStatus == KMMsgStatusWatched)
       mStatus &= ~KMMsgStatusIgnored;
-    if (aStatus == KMMsgStatusIgnored) {
+    if (aStatus == KMMsgStatusIgnored)
       mStatus &= ~KMMsgStatusWatched;
-      // Set to read. Don't use setStatus, that emits msgStatusChanged
-      // and we only want to do that once.
-      mStatus &= ~KMMsgStatusUnread;
-      mStatus &= ~KMMsgStatusNew;
-      mStatus |= KMMsgStatusRead;
-    }
     if (aStatus == KMMsgStatusSpam)
       mStatus &= ~KMMsgStatusHam;
     if (aStatus == KMMsgStatusHam)
@@ -252,7 +248,7 @@ void KMMsgBase::setStatus(const KMMsgStatus aStatus, int idx)
       break;
   }
 
-  if (storage()) {
+  if ( oldStatus != mStatus && storage() ) {
     if (idx < 0)
       idx = storage()->find( this );
     storage()->msgStatusChanged( oldStatus, status(), idx );
@@ -360,14 +356,14 @@ void KMMsgBase::setSignatureStateChar( QChar status, int idx )
 bool KMMsgBase::isUnread(void) const
 {
   KMMsgStatus st = status();
-  return (st & KMMsgStatusUnread);
+  return (st & KMMsgStatusUnread && !(st & KMMsgStatusIgnored));
 }
 
 //-----------------------------------------------------------------------------
 bool KMMsgBase::isNew(void) const
 {
   KMMsgStatus st = status();
-  return (st & KMMsgStatusNew);
+  return (st & KMMsgStatusNew && !(st & KMMsgStatusIgnored));
 }
 
 //-----------------------------------------------------------------------------
@@ -388,7 +384,7 @@ bool KMMsgBase::isOld(void) const
 bool KMMsgBase::isRead(void) const
 {
   KMMsgStatus st = status();
-  return (st & KMMsgStatusRead);
+  return (st & KMMsgStatusRead || st & KMMsgStatusIgnored);
 }
 
 //-----------------------------------------------------------------------------
@@ -570,7 +566,9 @@ QString KMMsgBase::skipKeyword(const QString& aStr, QChar sepChar,
 const QTextCodec* KMMsgBase::codecForName(const QCString& _str)
 {
   if (_str.isEmpty()) return 0;
-  return KGlobal::charsets()->codecForName(_str.lower());
+  QCString codec = _str;
+  KPIM::kAsciiToLower(codec.data());
+  return KGlobal::charsets()->codecForName(codec);
 }
 
 
@@ -648,7 +646,7 @@ namespace {
 
 
 //-----------------------------------------------------------------------------
-QString KMMsgBase::decodeRFC2047String(const QCString& aStr)
+QString KMMsgBase::decodeRFC2047String(const QCString& aStr, QCString prefCharset)
 {
   if ( aStr.isEmpty() )
     return QString::null;
@@ -658,13 +656,23 @@ QString KMMsgBase::decodeRFC2047String(const QCString& aStr)
   if ( str.isEmpty() )
     return QString::null;
 
-  if ( str.find( "=?" ) < 0 )
-    return kmkernel->networkCodec()->toUnicode( str );
+  if ( str.find( "=?" ) < 0 ) {
+    if ( !prefCharset.isEmpty() ) {
+      if ( prefCharset == "us-ascii" ) {
+        // isn`t this foolproof?
+        return KMMsgBase::codecForName( "utf-8" )->toUnicode( str );
+      } else {
+        return KMMsgBase::codecForName( prefCharset )->toUnicode( str );
+      }
+    } else {
+      return KMMsgBase::codecForName( GlobalSettings::self()->
+                                      fallbackCharacterEncoding().latin1() )->toUnicode( str );
+    }
+  }
 
   QString result;
   QCString LWSP_buffer;
   bool lastWasEncodedWord = false;
-  static const int maxLen = 200;
 
   for ( const char * pos = str.data() ; *pos ; ++pos ) {
     // collect LWSP after encoded-words,
@@ -687,11 +695,12 @@ QString KMMsgBase::decodeRFC2047String(const QCString& aStr)
       // parse charset name
       QCString charset;
       int i = 2;
-      for (pos+=2; i<maxLen && (*pos!='?'&&(*pos==' '||ispunct(*pos)||isalnum(*pos))); ++i) {
+      pos += 2;
+      for ( ; *pos != '?' && ( *pos==' ' || ispunct(*pos) || isalnum(*pos) );
+            ++i, ++pos ) {
 	charset += *pos;
-	pos++;
       }
-      if (*pos!='?' || i<4 || i>=maxLen)
+      if ( *pos!='?' || i<4 )
 	goto invalid_encoded_word;
 
       // get encoding and check delimiting question marks
@@ -702,11 +711,11 @@ QString KMMsgBase::decodeRFC2047String(const QCString& aStr)
       pos+=3; i+=3; // skip ?x?
       const char * enc_start = pos;
       // search for end of encoded part
-      while (i<maxLen && *pos && !(*pos=='?' && *(pos+1)=='=')) {
+      while ( *pos && !(*pos=='?' && *(pos+1)=='=') ) {
 	i++;
 	pos++;
       }
-      if (i>=maxLen || !*pos)
+      if ( !*pos )
 	goto invalid_encoded_word;
 
       // valid encoding: decode and throw away separating LWSP
@@ -762,8 +771,13 @@ QCString KMMsgBase::encodeRFC2047String(const QString& _str,
   if (charset == "us-ascii") return toUsAscii(_str);
 
   QCString cset;
-  if (charset.isEmpty()) cset = QCString(kmkernel->networkCodec()->mimeName()).lower();
-    else cset = charset;
+  if (charset.isEmpty())
+  {
+    cset = kmkernel->networkCodec()->mimeName();
+    KPIM::kAsciiToLower(cset.data());
+  }
+  else cset = charset;
+
   const QTextCodec *codec = codecForName(cset);
   if (!codec) codec = kmkernel->networkCodec();
 
@@ -849,7 +863,10 @@ QCString KMMsgBase::encodeRFC2231String( const QString& _str,
 
   QCString cset;
   if ( charset.isEmpty() )
-    cset = QCString( kmkernel->networkCodec()->mimeName() ).lower();
+  {
+    cset = kmkernel->networkCodec()->mimeName();
+    KPIM::kAsciiToLower( cset.data() );
+  }
   else
     cset = charset;
   const QTextCodec *codec = codecForName( cset );
@@ -863,7 +880,7 @@ QCString KMMsgBase::encodeRFC2231String( const QString& _str,
 
   char *l;
   for ( l = latin.data(); *l; ++l ) {
-    if ( ( *l & 0xE0 == 0 ) || ( *l & 0x80 ) )
+    if ( ( ( *l & 0xE0 ) == 0 ) || ( *l & 0x80 ) )
       // *l is control character or 8-bit char
       break;
   }
@@ -931,6 +948,42 @@ QString KMMsgBase::decodeRFC2231String(const QCString& _str)
   return codec->toUnicode( st );
 }
 
+QCString KMMsgBase::extractRFC2231HeaderField( const QCString &aStr, const QCString &field )
+{
+  int n=-1;
+  QCString str;
+  bool found = false;
+  while ( n<=0 || found ) {
+    QString pattern( field );
+    pattern += "[*]"; // match a literal * after the fieldname, as defined by RFC 2231
+    if ( n>=0 ) { // If n<0, check for fieldname*=..., otherwise for fieldname*n=
+      pattern += QString::number(n) + "[*]?";
+    }
+    pattern += "=";
+    
+    QRegExp fnamePart( pattern, FALSE );
+    int startPart = fnamePart.search( aStr );
+    int endPart;
+    found = ( startPart >= 0 );
+    if ( found ) {
+      startPart += fnamePart.matchedLength();
+      // Quoted values end at the ending quote
+      if ( aStr[startPart] == '"' ) {
+        startPart++; // the double quote isn't part of the filename
+        endPart = aStr.find('"', startPart) - 1;
+      }
+      else {
+        endPart = aStr.find(';', startPart) - 1;
+      }
+      if (endPart < 0)
+        endPart = 32767;
+      str += aStr.mid( startPart, endPart-startPart+1).stripWhiteSpace();
+    }
+    n++;
+  }
+  return str;
+}
+
 QString KMMsgBase::base64EncodedMD5( const QString & s, bool utf8 ) {
   if (s.stripWhiteSpace().isEmpty()) return "";
   if ( utf8 )
@@ -968,7 +1021,10 @@ QCString KMMsgBase::autoDetectCharset(const QCString &_encoding, const QStringLi
     {
        QCString encoding = (*it).latin1();
        if (encoding == "locale")
-          encoding = QCString(kmkernel->networkCodec()->mimeName()).lower();
+       {
+         encoding = kmkernel->networkCodec()->mimeName();
+         KPIM::kAsciiToLower(encoding.data());
+       }
        if (text.isEmpty())
          return encoding;
        if (encoding == "us-ascii") {
@@ -1000,7 +1056,7 @@ unsigned long KMMsgBase::getMsgSerNum() const
     return msn;
   if (mParent) {
     int index = mParent->find((KMMsgBase*)this);
-    msn = kmkernel->msgDict()->getMsgSerNum(mParent, index);
+    msn = KMMsgDict::instance()->getMsgSerNum(mParent, index);
     if (msn)
       MessageProperty::setSerialCache( this, msn );
   }
@@ -1008,49 +1064,6 @@ unsigned long KMMsgBase::getMsgSerNum() const
 }
 
 
-//-----------------------------------------------------------------------------
-bool KMMsgBase::isComplete()
-{
-  return MessageProperty::complete( getMsgSerNum() );
-}
-
-
-//-----------------------------------------------------------------------------
-void KMMsgBase::setComplete(bool value)
-{
-  MessageProperty::setComplete( getMsgSerNum(), value );
-  if ( value )
-    setReadyToShow( true );
-}
-
-//-----------------------------------------------------------------------------
-bool KMMsgBase::readyToShow()
-{
-  return MessageProperty::readyToShow( getMsgSerNum() );
-}
-
-
-//-----------------------------------------------------------------------------
-void KMMsgBase::setReadyToShow(bool value)
-{
-  MessageProperty::setReadyToShow( getMsgSerNum(), value );
-}
-
-
-//-----------------------------------------------------------------------------
-bool KMMsgBase::transferInProgress()
-{
-  return MessageProperty::transferInProgress( getMsgSerNum() );
-}
-
-
-//-----------------------------------------------------------------------------
-void KMMsgBase::setTransferInProgress(bool value, bool force)
-{
-  MessageProperty::setTransferInProgress( getMsgSerNum(), value, force );
-}
-
-
 //-----------------------------------------------------------------------------
 KMMsgAttachmentState KMMsgBase::attachmentState() const
 {
