@@ -371,7 +371,7 @@ kdDebug(5006) << "dataNode == curNode:  Save curNode without replacing it." << e
         if( dataNode->parentNode() && !weAreReplacingTheRootNode ) {
 kdDebug(5006) << "dataNode is NOT replacing the root node:  Store the headers." << endl;
           resultingData += headers->AsString().c_str();
-        } else if( weAreReplacingTheRootNode && part->hasHeaders() ){
+        } else if( weAreReplacingTheRootNode && part && part->hasHeaders() ){
 kdDebug(5006) << "dataNode replace the root node:  Do NOT store the headers but change" << endl;
 kdDebug(5006) << "                                 the Message's headers accordingly." << endl;
 kdDebug(5006) << "              old Content-Type = " << rootHeaders.ContentType().AsString().c_str() << endl;
@@ -843,6 +843,10 @@ void KMReaderWin::update( KMail::Interface::Observable * observable )
   assert( msg != 0 );
 
   // find our partNode and update it
+  if ( !msg->lastUpdatedPart() ) {
+    kdDebug(5006) << "KMReaderWin::update - no updated part" << endl;
+    return;
+  }
   partNode* node = mRootNode->findNodeForDwPart( msg->lastUpdatedPart() );
   if ( !node ) {
     kdDebug(5006) << "KMReaderWin::update - can't find node for part" << endl;
@@ -860,6 +864,8 @@ void KMReaderWin::update( KMail::Interface::Observable * observable )
   }
   KPIM::kBytesToFile( data.data(), size, mAtmCurrentName, false, false, false );
   ::chmod( QFile::encodeName( mAtmCurrentName ), S_IRUSR );
+
+  mAtmUpdate = false;
 }
 
 //-----------------------------------------------------------------------------
@@ -1090,13 +1096,20 @@ void KMReaderWin::setOverrideEncoding( const QString & encoding )
     }
     else {
       QStringList encodings = mSelectEncodingAction->items();
-      int i = 0;
+      uint i = 0;
       for ( QStringList::const_iterator it = encodings.begin(), end = encodings.end(); it != end; ++it, ++i ) {
         if ( KGlobal::charsets()->encodingForName( *it ) == encoding ) {
           mSelectEncodingAction->setCurrentItem( i );
           break;
         }
       }
+      if ( i == encodings.size() ) {
+        // the value of encoding is unknown => use Auto
+        kdWarning(5006) << "Unknown override character encoding \"" << encoding
+                        << "\". Using Auto instead." << endl;
+        mSelectEncodingAction->setCurrentItem( 0 );
+        mOverrideEncoding = QString::null;
+      }
     }
   }
   update( true );
@@ -1935,6 +1948,12 @@ void KMReaderWin::slotFind()
   mViewer->findText();
 }
 
+//-----------------------------------------------------------------------------
+void KMReaderWin::slotFindNext()
+{
+  mViewer->findTextNext();
+}
+
 //-----------------------------------------------------------------------------
 void KMReaderWin::slotToggleFixedFont()
 {
@@ -1957,15 +1976,8 @@ void KMReaderWin::slotCopySelectedText()
 void KMReaderWin::atmViewMsg(KMMessagePart* aMsgPart)
 {
   assert(aMsgPart!=0);
-  partNode* node = mRootNode ? mRootNode->findId( mAtmCurrent ) : 0;
-  KMMessage* msg;
-  if (node && node->dwPart()->Body().Message()) {
-    // make a deep copy
-    msg = new KMMessage( new DwMessage(*node->dwPart()->Body().Message()) );
-  } else {
-    msg = new KMMessage;
-    msg->fromString(aMsgPart->bodyDecoded());
-  }
+  KMMessage* msg = new KMMessage;
+  msg->fromString(aMsgPart->bodyDecoded());
   assert(msg != 0);
   msg->setMsgSerNum( 0 ); // because lookups will fail
   // some information that is needed for imap messages with LOD
@@ -2068,16 +2080,18 @@ void KMReaderWin::setMsgPart( KMMessagePart* aMsgPart, bool aHTML,
   } else {
     htmlWriter()->begin( mCSSHelper->cssDefinitions( isFixedFont() ) );
     htmlWriter()->queue( mCSSHelper->htmlHead( isFixedFont() ) );
+    htmlWriter()->queue( "<pre>" );
 
     QString str = aMsgPart->bodyDecoded();
     // A QString cannot handle binary data. So if it's shorter than the
     // attachment, we assume the attachment is binary:
     if( str.length() < (unsigned) aMsgPart->decodedSize() ) {
-      str += QString::fromLatin1("\n") + i18n("[KMail: Attachment contains binary data. Trying to show first character.]",
+      str.prepend( i18n("[KMail: Attachment contains binary data. Trying to show first character.]",
           "[KMail: Attachment contains binary data. Trying to show first %n characters.]",
-          str.length());
+          str.length()) + QChar('\n') );
     }
-    htmlWriter()->write( QStyleSheet::escape( str ) );
+    htmlWriter()->queue( QStyleSheet::escape( str ) );
+    htmlWriter()->queue( "</pre>" );
     htmlWriter()->queue("</body></html>");
     htmlWriter()->flush();
     mMainWindow->setCaption(i18n("View Attachment: %1").arg(pname));
