@@ -36,8 +36,8 @@
 #include <kmessagebox.h>
 #include <kinputdialog.h>
 #include <kio/netaccess.h>
+#include <kabc/addressee.h>
 
-#include <libkdepim/categoryselectdialog.h>
 #include <libkdepim/designerfields.h>
 #include <libkdepim/embeddedurlpage.h>
 
@@ -73,19 +73,12 @@ KOIncidenceEditor::KOIncidenceEditor( const QString &caption,
     setButtonText( Default, i18n("&Templates...") );
   }
 
-  mCategoryDialog = new KPIM::CategorySelectDialog( KOPrefs::instance(), this );
-  KOGlobals::fitDialogToScreen( mCategoryDialog );
-
-  connect( mCategoryDialog, SIGNAL( editCategories() ),
-           SIGNAL( editCategories() ) );
-
   connect( this, SIGNAL( defaultClicked() ), SLOT( slotManageTemplates() ) );
   connect( this, SIGNAL( finished() ), SLOT( delayedDestruct() ) );
 }
 
 KOIncidenceEditor::~KOIncidenceEditor()
 {
-  delete mCategoryDialog;
 }
 
 void KOIncidenceEditor::setupAttendeesTab()
@@ -129,7 +122,6 @@ void KOIncidenceEditor::slotOk()
 
 void KOIncidenceEditor::updateCategoryConfig()
 {
-  mCategoryDialog->updateCategoryConfig();
 }
 
 void KOIncidenceEditor::slotCancel()
@@ -160,8 +152,6 @@ void KOIncidenceEditor::slotManageTemplates()
 {
   kdDebug(5850) << "KOIncidenceEditor::manageTemplates()" << endl;
 
-  QString tp = type();
-
   TemplateManagementDialog * const d = new TemplateManagementDialog( this, templates() );
   connect( d, SIGNAL( loadTemplate( const QString& ) ),
            this, SLOT( slotLoadTemplate( const QString& ) ) );
@@ -378,4 +368,24 @@ void KOIncidenceEditor::openURL( const KURL &url )
   UriHandler::process( uri );
 }
 
+void KOIncidenceEditor::addAttachments( const QStringList &attachments )
+{
+  QStringList::ConstIterator it;
+  for ( it = attachments.begin(); it != attachments.end(); ++it ) {
+    mAttachments->addAttachment( *it );
+  }
+}
+
+void KOIncidenceEditor::addAttendees( const QStringList &attendees )
+{
+  QStringList::ConstIterator it;
+  for ( it = attendees.begin(); it != attendees.end(); ++it ) {
+    QString name, email;
+    KABC::Addressee::parseEmailAddress( *it, name, email );
+    mDetails->insertAttendee( new Attendee( name, email ) );
+  }
+}
+
+
+
 #include "koincidenceeditor.moc"
