@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -25,6 +26,8 @@
 #include <qlayout.h>
 #include <qheader.h>
 #include <qcursor.h>
+#include <qlabel.h>
+#include <qtimer.h>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -37,20 +40,28 @@
 #include <libkcal/dndfactory.h>
 #include <libkcal/calendarresources.h>
 #include <libkcal/resourcecalendar.h>
+#include <libkcal/calfilter.h>
+#include <libkcal/incidenceformatter.h>
+
+#include <libkdepim/clicklineedit.h>
+#include <libkdepim/kdatepickerpopup.h>
+
+#include <libemailfunctions/email.h>
 
-#ifndef KORG_NOPRINTER
-#include "calprinter.h"
-#endif
 #include "docprefs.h"
 
 #include "koincidencetooltip.h"
 #include "kodialogmanager.h"
 #include "kotodoview.h"
+#include "koprefs.h"
 #include "koglobals.h"
 using namespace KOrg;
+#include "kotodoviewitem.h"
 #include "kotodoview.moc"
-
-const int KOTodoView::POPUP_UNSUBTODO=1234;
+#ifndef KORG_NOPRINTER
+#include "kocorehelper.h"
+#include "calprinter.h"
+#endif
 
 KOTodoListViewToolTip::KOTodoListViewToolTip (QWidget* parent,
                                               KOTodoListView* lv )
@@ -77,9 +88,8 @@ void KOTodoListViewToolTip::maybeTip( const QPoint & pos)
     r.setRight(headerPos + todolist->header()->sectionSize(col));
 
     /* Show the tip */
-    QString tipText;
-    ToolTipVisitor v;
-    if (v.act(i->todo(), &tipText, true)) {
+    QString tipText( IncidenceFormatter::toolTipString( i->todo() ) );;
+    if ( !tipText.isEmpty() ) {
       tip(r, tipText);
     }
   }
@@ -89,7 +99,7 @@ void KOTodoListViewToolTip::maybeTip( const QPoint & pos)
 
 
 KOTodoListView::KOTodoListView( QWidget *parent, const char *name )
-  : KListView( parent, name ), mCalendar( 0 )
+  : KListView( parent, name ), mCalendar( 0 ), mChanger( 0 )
 {
   mOldCurrent = 0;
   mMousePressed = false;
@@ -149,7 +159,6 @@ void KOTodoListView::contentsDragEnterEvent(QDragEnterEvent *e)
 #endif
 }
 
-
 void KOTodoListView::contentsDragMoveEvent(QDragMoveEvent *e)
 {
 #ifndef KORG_NODND
@@ -178,9 +187,9 @@ void KOTodoListView::contentsDragLeaveEvent( QDragLeaveEvent * )
 void KOTodoListView::contentsDropEvent( QDropEvent *e )
 {
 #ifndef KORG_NODND
-//  kdDebug(5850) << "KOTodoListView::contentsDropEvent" << endl;
+  kdDebug(5850) << "KOTodoListView::contentsDropEvent" << endl;
 
-  if ( !mCalendar ||
+  if ( !mCalendar || !mChanger ||
        ( !ICalDrag::canDecode( e ) && !VCalDrag::canDecode( e ) &&
          !QTextDrag::canDecode( e ) ) ) {
     e->ignore();
@@ -190,7 +199,7 @@ void KOTodoListView::contentsDropEvent( QDropEvent *e )
   DndFactory factory( mCalendar );
   Todo *todo = factory.createDropTodo(e);
 
-  if (todo) {
+  if ( todo ) {
     e->acceptAction();
 
     KOTodoViewItem *destination =
@@ -200,53 +209,75 @@ void KOTodoListView::contentsDropEvent( QDropEvent *e )
 
     Todo *existingTodo = mCalendar->todo(todo->uid());
 
-    if(existingTodo) {
-//      kdDebug(5850) << "Drop existing Todo" << endl;
+    if( existingTodo ) {
+       kdDebug(5850) << "Drop existing Todo " << existingTodo << " onto " << destinationEvent << endl;
       Incidence *to = destinationEvent;
       while(to) {
         if (to->uid() == todo->uid()) {
-          KMessageBox::sorry(this,
-              i18n("Cannot move To-Do to itself or a child of itself"),
-              i18n("Drop To-Do"));
+          KMessageBox::information(this,
+              i18n("Cannot move to-do to itself or a child of itself."),
+              i18n("Drop To-do"), "NoDropTodoOntoItself" );
           delete todo;
           return;
         }
         to = to->relatedTo();
       }
       Todo*oldTodo = existingTodo->clone();
-      existingTodo->setRelatedTo(destinationEvent);
-
-      emit todoDropped( todo );
-      emit todoChanged( oldTodo, todo );
+      if ( mChanger->beginChange( existingTodo ) ) {
+        existingTodo->setRelatedTo( destinationEvent );
+        mChanger->changeIncidence( oldTodo, existingTodo, KOGlobals::RELATION_MODIFIED );
+        mChanger->endChange( existingTodo );
+      } else {
+        KMessageBox::sorry( this, i18n("Unable to change to-do's parent, "
+                            "because the to-do cannot be locked.") );
+      }
       delete oldTodo;
       delete todo;
     } else {
 //      kdDebug(5850) << "Drop new Todo" << endl;
       todo->setRelatedTo(destinationEvent);
-      if ( !mCalendar->addTodo( todo ) ) {
-        KODialogManager::errorSaveTodo( this );
+      if ( !mChanger->addIncidence( todo, this ) ) {
+        KODialogManager::errorSaveIncidence( this, todo );
+        delete todo;
         return;
       }
-
-      emit todoDropped(todo);
-      emit todoAdded( todo );
     }
   }
   else {
     QString text;
-    if (QTextDrag::decode(e,text)) {
+    KOTodoViewItem *todoi = dynamic_cast<KOTodoViewItem *>(itemAt( contentsToViewport(e->pos()) ));
+    if ( ! todoi ) {
+      // Not dropped on a todo item:
+      e->ignore();
+      kdDebug( 5850 ) << "KOTodoListView::contentsDropEvent(): Not dropped on a todo item" << endl;
+      kdDebug( 5850 ) << "TODO: Create a new todo with the given data" << endl;
+      // FIXME: Create a new todo with the given text/contact/whatever
+    } else if ( QTextDrag::decode(e, text) ) {
       //QListViewItem *qlvi = itemAt( contentsToViewport(e->pos()) );
-      KOTodoViewItem *todoi = static_cast<KOTodoViewItem *>(itemAt( contentsToViewport(e->pos()) ));
       kdDebug(5850) << "Dropped : " << text << endl;
-      QStringList emails = QStringList::split(",",text);
-      for(QStringList::ConstIterator it = emails.begin();it!=emails.end();++it) {
-        kdDebug(5850) << " Email: " << (*it) << endl;
-        int pos = (*it).find("<");
-        QString name = (*it).left(pos);
-        QString email = (*it).mid(pos);
-        if (!email.isEmpty() && todoi) {
-          todoi->todo()->addAttendee(new Attendee(name,email));
+      Todo*todo = todoi->todo();
+      if( mChanger->beginChange( todo ) ) {
+        Todo*oldtodo = todo->clone();
+
+        if( text.startsWith( "file:" ) ) {
+          todo->addAttachment( new Attachment( text ) );
+        } else {
+          QStringList emails = KPIM::splitEmailAddrList( text );
+          for(QStringList::ConstIterator it = emails.begin();it!=emails.end();++it) {
+            kdDebug(5850) << " Email: " << (*it) << endl;
+            int pos = (*it).find("<");
+            QString name = (*it).left(pos);
+            QString email = (*it).mid(pos);
+            if (!email.isEmpty() && todoi) {
+              todo->addAttendee( new Attendee( name, email ) );
+            }
+          }
         }
+        mChanger->changeIncidence( oldtodo, todo );
+        mChanger->endChange( todo );
+      } else {
+        KMessageBox::sorry( this, i18n("Unable to add attendees to the to-do, "
+            "because the to-do cannot be locked.") );
       }
     }
     else {
@@ -330,41 +361,17 @@ void KOTodoListView::contentsMouseDoubleClickEvent(QMouseEvent *e)
 
 /////////////////////////////////////////////////////////////////////////////
 
-KOQuickTodo::KOQuickTodo(QWidget *parent) :
-  QLineEdit(parent)
-{
-  setText(i18n("Click to add a new Todo"));
-  setPaletteForegroundColor(gray);
-}
-
-void KOQuickTodo::focusInEvent(QFocusEvent *ev)
-{
-  if ( text()==i18n("Click to add a new Todo") )
-    setText(QString::null);
-  setPaletteForegroundColor(parentWidget()->paletteForegroundColor());
-  QLineEdit::focusInEvent(ev);
-}
-
-void KOQuickTodo::focusOutEvent(QFocusEvent *ev)
-{
-  if ( text().isEmpty() ) 
-	  setText(i18n("Click to add a new Todo"));
-  setPaletteForegroundColor(gray);
-  QLineEdit::focusOutEvent(ev);
-}
-
-/////////////////////////////////////////////////////////////////////////////
-
 KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   : KOrg::BaseView( calendar, parent, name )
 {
   QBoxLayout *topLayout = new QVBoxLayout( this );
 
-  QLabel *title = new QLabel( i18n("To-do items:"), this );
+  QLabel *title = new QLabel( i18n("To-dos:"), this );
   title->setFrameStyle( QFrame::Panel | QFrame::Raised );
   topLayout->addWidget( title );
 
-  mQuickAdd = new KOQuickTodo( this );
+  mQuickAdd = new KPIM::ClickLineEdit( this, i18n( "Click to add a new to-do" ) );
+  mQuickAdd->setAcceptDrops( false );
   topLayout->addWidget( mQuickAdd );
 
   if ( !KOPrefs::instance()->mEnableQuickTodo ) mQuickAdd->hide();
@@ -378,14 +385,13 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   mTodoListView->setShowSortIndicator( true );
 
   mTodoListView->addColumn( i18n("Summary") );
+  mTodoListView->addColumn( i18n("Recurs") );
   mTodoListView->addColumn( i18n("Priority") );
-  mTodoListView->setColumnAlignment( 1, AlignHCenter );
+  mTodoListView->setColumnAlignment( ePriorityColumn, AlignHCenter );
   mTodoListView->addColumn( i18n("Complete") );
-  mTodoListView->setColumnAlignment( 2, AlignRight );
-  mTodoListView->addColumn( i18n("Due Date") );
-  mTodoListView->setColumnAlignment( 3, AlignHCenter );
-  mTodoListView->addColumn( i18n("Due Time") );
-  mTodoListView->setColumnAlignment( 4, AlignHCenter );
+  mTodoListView->setColumnAlignment( ePercentColumn, AlignRight );
+  mTodoListView->addColumn( i18n("Due Date/Time") );
+  mTodoListView->setColumnAlignment( eDueDateColumn, AlignLeft );
   mTodoListView->addColumn( i18n("Categories") );
 #if 0
   mTodoListView->addColumn( i18n("Sort Id") );
@@ -396,54 +402,88 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   mTodoListView->setItemsRenameable( true );
   mTodoListView->setRenameable( 0 );
 
-  mTodoListView->setColumnWidthMode( 0, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 1, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 2, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 3, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 4, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 5, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eSummaryColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eRecurColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( ePriorityColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( ePercentColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eDueDateColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eCategoriesColumn, QListView::Manual );
 #if 0
-  mTodoListView->setColumnWidthMode( 6, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eDescriptionColumn, QListView::Manual );
 #endif
 
   mPriorityPopupMenu = new QPopupMenu( this );
-  for ( int i = 1; i <= 5; i++ ) {
-    QString label = QString ("%1").arg( i );
-    mPriority[ mPriorityPopupMenu->insertItem( label ) ] = i;
-  }
+  mPriority[ mPriorityPopupMenu->insertItem( i18n("Unspecified priority", "unspecified") ) ] = 0;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "1 (highest)") ) ] = 1;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "2" ) ) ] = 2;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "3" ) ) ] = 3;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "4" ) ) ] = 4;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "5 (medium)" ) ) ] = 5;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "6" ) ) ] = 6;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "7" ) ) ] = 7;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "8" ) ) ] = 8;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "9 (lowest)" ) ) ] = 9;
   connect( mPriorityPopupMenu, SIGNAL( activated( int ) ),
            SLOT( setNewPriority( int ) ));
 
   mPercentageCompletedPopupMenu = new QPopupMenu(this);
-  for (int i = 0; i <= 100; i+=20) {
+  for (int i = 0; i <= 100; i+=10) {
     QString label = QString ("%1 %").arg (i);
     mPercentage[mPercentageCompletedPopupMenu->insertItem (label)] = i;
   }
   connect( mPercentageCompletedPopupMenu, SIGNAL( activated( int ) ),
            SLOT( setNewPercentage( int ) ) );
 
+  mMovePopupMenu = new KDatePickerPopup(
+                             KDatePickerPopup::NoDate |
+                             KDatePickerPopup::DatePicker |
+                             KDatePickerPopup::Words );
+  mCopyPopupMenu = new KDatePickerPopup(
+                             KDatePickerPopup::NoDate |
+                             KDatePickerPopup::DatePicker |
+                             KDatePickerPopup::Words );
+
+
+  connect( mMovePopupMenu, SIGNAL( dateChanged( QDate )),
+           SLOT( setNewDate( QDate ) ) );
+  connect( mCopyPopupMenu, SIGNAL( dateChanged( QDate )),
+           SLOT( copyTodoToDate( QDate ) ) );
+
   mItemPopupMenu = new QPopupMenu(this);
-  mItemPopupMenu->insertItem(i18n("Show"), this,
+  mItemPopupMenu->insertItem(i18n("&Show"), this,
                              SLOT (showTodo()));
-  mItemPopupMenu->insertItem(i18n("Edit..."), this,
-                             SLOT (editTodo()));
-  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("editdelete"), i18n("Delete"), this,
-                             SLOT (deleteTodo()));
+  mItemPopupMenu->insertItem(i18n("&Edit..."), this,
+                             SLOT (editTodo()), 0, ePopupEdit );
+#ifndef KORG_NOPRINTER
+  mItemPopupMenu->insertItem(KOGlobals::self()->smallIcon("printer1"), i18n("&Print..."), this, SLOT( printTodo() ) );
+#endif
+  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("editdelete"), i18n("&Delete"), this,
+                             SLOT (deleteTodo()), 0, ePopupDelete );
   mItemPopupMenu->insertSeparator();
-  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-Do..."), this,
+  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New &To-do..."), this,
                              SLOT (newTodo()));
-  mItemPopupMenu->insertItem(i18n("New Sub-To-Do..."), this,
+  mItemPopupMenu->insertItem(i18n("New Su&b-to-do..."), this,
                              SLOT (newSubTodo()));
-  mItemPopupMenu->insertItem( i18n("Make Sub-To-Do Independent"), this,
-      SIGNAL( unSubTodoSignal() ), 0, POPUP_UNSUBTODO );
+  mItemPopupMenu->insertItem( i18n("&Make this To-do Independent"), this,
+      SIGNAL( unSubTodoSignal() ), 0, ePopupUnSubTodo );
+  mItemPopupMenu->insertItem( i18n("Make all Sub-to-dos &Independent"), this,
+      SIGNAL( unAllSubTodoSignal() ), 0, ePopupUnAllSubTodo );
   mItemPopupMenu->insertSeparator();
-  mItemPopupMenu->insertItem(i18n("delete completed To-Dos","Purge Completed"),
+  mItemPopupMenu->insertItem( i18n("&Copy To"), mCopyPopupMenu, ePopupCopyTo );
+  mItemPopupMenu->insertItem(i18n("&Move To"), mMovePopupMenu, ePopupMoveTo );
+  mItemPopupMenu->insertSeparator();
+  mItemPopupMenu->insertItem(i18n("delete completed to-dos","Pur&ge Completed"),
                              this, SLOT( purgeCompleted() ) );
 
+  connect( mMovePopupMenu, SIGNAL( dateChanged( QDate ) ),
+           mItemPopupMenu, SLOT( hide() ) );
+  connect( mCopyPopupMenu, SIGNAL( dateChanged( QDate ) ),
+           mItemPopupMenu, SLOT( hide() ) );
+
   mPopupMenu = new QPopupMenu(this);
-  mPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-Do..."), this,
+  mPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("&New To-do..."), this,
                          SLOT (newTodo()));
-  mPopupMenu->insertItem(i18n("delete completed To-Dos","Purge Completed"),
+  mPopupMenu->insertItem(i18n("delete completed to-dos","&Purge Completed"),
                          this, SLOT(purgeCompleted()));
 
   mDocPrefs = new DocPrefs( name );
@@ -457,10 +497,6 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   connect( mTodoListView, SIGNAL( contextMenuRequested( QListViewItem *,
                                                         const QPoint &, int ) ),
            SLOT( popupMenu( QListViewItem *, const QPoint &, int ) ) );
-  connect( mTodoListView, SIGNAL( clicked( QListViewItem * ) ),
-           SLOT( itemClicked( QListViewItem * ) ) );
-  connect( mTodoListView, SIGNAL( todoDropped( Todo * ) ),
-           SLOT( updateView() ) );
   connect( mTodoListView, SIGNAL( expanded( QListViewItem * ) ),
            SLOT( itemStateChanged( QListViewItem * ) ) );
   connect( mTodoListView, SIGNAL( collapsed( QListViewItem * ) ),
@@ -478,10 +514,6 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
            SLOT( processSelectionChange() ) );
   connect( mQuickAdd, SIGNAL( returnPressed () ),
            SLOT( addQuickTodo() ) );
-  connect( mTodoListView, SIGNAL( todoChanged( Todo*, Todo* ) ),
-           SIGNAL( todoChanged( Todo*, Todo* ) ) );
-  connect( mTodoListView, SIGNAL( todoAdded( Todo* ) ),
-           SIGNAL( todoAdded( Todo* ) ) );
 }
 
 KOTodoView::~KOTodoView()
@@ -498,7 +530,8 @@ void KOTodoView::setCalendar( Calendar *cal )
 void KOTodoView::updateView()
 {
 //  kdDebug(5850) << "KOTodoView::updateView()" << endl;
-
+  int oldPos = mTodoListView->contentsY();
+  mItemsToDelete.clear();
   mTodoListView->clear();
 
   Todo::List todoList = calendar()->todos();
@@ -537,6 +570,8 @@ void KOTodoView::updateView()
   if( mDocPrefs ) restoreItemState( mTodoListView->firstChild() );
   mTodoListView->blockSignals( false );
 
+  mTodoListView->setContentsPos( 0, oldPos );
+
   processSelectionChange();
 }
 
@@ -555,10 +590,13 @@ QMap<Todo *,KOTodoViewItem *>::ConstIterator
   KOTodoView::insertTodoItem(Todo *todo)
 {
 //  kdDebug(5850) << "KOTodoView::insertTodoItem(): " << todo->getSummary() << endl;
-  // TODO: Check, if dynmaic cast is necessary
   Incidence *incidence = todo->relatedTo();
   if (incidence && incidence->type() == "Todo") {
-    Todo *relatedTodo = static_cast<Todo *>(incidence);
+    // Use dynamic_cast, because in the future the related item might also be an event
+    Todo *relatedTodo = dynamic_cast<Todo *>(incidence);
+
+    // just make sure we know we have this item already to avoid endless recursion (Bug 101696)
+    mTodoMap.insert(todo,0);
 
 //    kdDebug(5850) << "  has Related" << endl;
     QMap<Todo *,KOTodoViewItem *>::ConstIterator itemIterator;
@@ -569,7 +607,17 @@ QMap<Todo *,KOTodoViewItem *>::ConstIterator
     }
     // isn't this pretty stupid? We give one Todo  to the KOTodoViewItem
     // and one into the map. Sure finding is more easy but why? -zecke
-    KOTodoViewItem *todoItem = new KOTodoViewItem(*itemIterator,todo,this);
+    KOTodoViewItem *todoItem;
+
+    // in case we found a related parent, which has no KOTodoViewItem yet, this must
+    // be the case where 2 items refer to each other, therefore simply create item as root item
+    if ( *itemIterator == 0 ) {
+      todo->setRelatedTo(0);  // break the recursion, else we will have troubles later
+      todoItem = new KOTodoViewItem(mTodoListView,todo,this);
+    }
+    else
+      todoItem = new KOTodoViewItem(*itemIterator,todo,this);
+
     return mTodoMap.insert(todo,todoItem);
   } else {
 //    kdDebug(5850) << "  no Related" << endl;
@@ -579,6 +627,29 @@ QMap<Todo *,KOTodoViewItem *>::ConstIterator
   }
 }
 
+void KOTodoView::removeTodoItems()
+{
+  KOTodoViewItem *item;
+  for ( item = mItemsToDelete.first(); item; item = mItemsToDelete.next() ) {
+    Todo *todo = item->todo();
+    if ( todo && mTodoMap.contains( todo ) ) {
+      mTodoMap.remove( todo );
+    }
+    delete item;
+  }
+  mItemsToDelete.clear();
+}
+
+
+bool KOTodoView::scheduleRemoveTodoItem( KOTodoViewItem *todoItem )
+{
+  if ( todoItem ) {
+    mItemsToDelete.append( todoItem );
+    QTimer::singleShot( 0, this, SLOT( removeTodoItems() ) );
+    return true;
+  } else
+    return false;
+}
 
 void KOTodoView::updateConfig()
 {
@@ -607,36 +678,88 @@ Todo::List KOTodoView::selectedTodos()
   return selected;
 }
 
-void KOTodoView::changeEventDisplay(Event *, int)
+void KOTodoView::changeIncidenceDisplay(Incidence *incidence, int action)
 {
-  updateView();
+  // The todo view only displays todos, so exit on all other incidences
+  if ( incidence->type() != "Todo" )
+    return;
+  CalFilter *filter = calendar()->filter();
+  bool isFiltered = filter && !filter->filterIncidence( incidence );
+  Todo *todo = static_cast<Todo *>(incidence);
+  if ( todo ) {
+    KOTodoViewItem *todoItem = 0;
+    if ( mTodoMap.contains( todo ) ) {
+      todoItem = mTodoMap[todo];
+    }
+    switch ( action ) {
+      case KOGlobals::INCIDENCEADDED:
+      case KOGlobals::INCIDENCEEDITED:
+        // If it's already there, edit it, otherwise just add
+        if ( todoItem ) {
+          if ( isFiltered ) {
+            scheduleRemoveTodoItem( todoItem );
+          } else {
+            // correctly update changes in relations
+            Todo*parent = dynamic_cast<Todo*>( todo->relatedTo() );
+            KOTodoViewItem*parentItem = 0;
+            if ( parent && mTodoMap.contains(parent) ) {
+              parentItem = mTodoMap[ parent ];
+            }
+            if ( todoItem->parent() != parentItem ) {
+              // The relations changed
+              if ( parentItem ) {
+                parentItem->insertItem( todoItem );
+              } else {
+                mTodoListView->insertItem( todoItem );
+              }
+            }
+            todoItem->construct();
+          }
+        } else {
+          if ( !isFiltered ) {
+            insertTodoItem( todo );
+          }
+        }
+        mTodoListView->sort();
+        break;
+      case KOGlobals::INCIDENCEDELETED:
+        if ( todoItem ) {
+          scheduleRemoveTodoItem( todoItem );
+        }
+        break;
+      default:
+        QTimer::singleShot( 0, this, SLOT( updateView() ) );
+    }
+  } else {
+    // use a QTimer here, because when marking todos finished using
+    // the checkbox, this slot gets called, but we cannot update the views
+    // because we're still inside KOTodoViewItem::stateChange
+    QTimer::singleShot(0,this,SLOT(updateView()));
+  }
 }
 
 void KOTodoView::showDates(const QDate &, const QDate &)
 {
 }
 
-void KOTodoView::showEvents( const Event::List & )
+void KOTodoView::showIncidences( const Incidence::List & )
 {
-  kdDebug(5850) << "KOTodoView::selectEvents(): not yet implemented" << endl;
+  kdDebug(5850) << "KOTodoView::showIncidences( const Incidence::List & ): not yet implemented" << endl;
 }
 
-void KOTodoView::printPreview(CalPrinter *calPrinter, const QDate &fd,
-                              const QDate &td)
+CalPrinterBase::PrintType KOTodoView::printType()
 {
-#ifndef KORG_NOPRINTER
-  calPrinter->preview(CalPrinter::Todolist, fd, td);
-#endif
-}
-
-CalPrinter::PrintType KOTodoView::printType()
-{
-  return CalPrinter::Todolist;
+  if ( mTodoListView->selectedItem() ) {
+    return CalPrinterBase::Incidence;
+  } else {
+    return CalPrinterBase::Todolist;
+  }
 }
 
 void KOTodoView::editItem( QListViewItem *item )
 {
-  emit editTodoSignal( static_cast<KOTodoViewItem *>( item )->todo() );
+  if (item)
+    emit editIncidenceSignal( static_cast<KOTodoViewItem *>( item )->todo() );
 }
 
 void KOTodoView::editItem( QListViewItem *item, const QPoint &, int )
@@ -644,37 +767,69 @@ void KOTodoView::editItem( QListViewItem *item, const QPoint &, int )
   editItem( item );
 }
 
+void KOTodoView::showItem( QListViewItem *item )
+{
+  if (item)
+    emit showIncidenceSignal( static_cast<KOTodoViewItem *>( item )->todo() );
+}
+
 void KOTodoView::showItem( QListViewItem *item, const QPoint &, int )
 {
-  emit showTodoSignal( static_cast<KOTodoViewItem *>( item )->todo() );
+  showItem( item );
 }
 
 void KOTodoView::popupMenu( QListViewItem *item, const QPoint &, int column )
 {
   mActiveItem = static_cast<KOTodoViewItem *>( item );
-  if ( item ) {
-    switch ( column ) {
-      case 1:
-        mPriorityPopupMenu->popup( QCursor::pos () );
-        break;
-      case 2:
-        mPercentageCompletedPopupMenu->popup( QCursor::pos () );
-        break;
-      case 5:
-        getCategoryPopupMenu(
-            static_cast<KOTodoViewItem *>( item ) )->popup( QCursor::pos () );
-        break;
-      default:
-        mItemPopupMenu->setItemEnabled( POPUP_UNSUBTODO,
-                                        mActiveItem->todo()->relatedTo() );
-        mItemPopupMenu->popup( QCursor::pos() );
+  if ( mActiveItem && mActiveItem->todo() &&
+       !mActiveItem->todo()->isReadOnly() ) {
+    bool editable = !mActiveItem->todo()->isReadOnly();
+    mItemPopupMenu->setItemEnabled( ePopupEdit, editable );
+    mItemPopupMenu->setItemEnabled( ePopupDelete, editable );
+    mItemPopupMenu->setItemEnabled( ePopupMoveTo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupCopyTo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupUnSubTodo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupUnAllSubTodo, editable );
+
+    if ( editable ) {
+      QDate date = mActiveItem->todo()->dtDue().date();
+      if ( mActiveItem->todo()->hasDueDate () ) {
+        mMovePopupMenu->datePicker()->setDate( date );
+      } else {
+        mMovePopupMenu->datePicker()->setDate( QDate::currentDate() );
+      }
+      switch ( column ) {
+        case ePriorityColumn:
+          mPriorityPopupMenu->popup( QCursor::pos() );
+          break;
+        case ePercentColumn: {
+          mPercentageCompletedPopupMenu->popup( QCursor::pos() );
+          break;
+        }
+        case eDueDateColumn:
+          mMovePopupMenu->popup( QCursor::pos() );
+          break;
+        case eCategoriesColumn:
+          getCategoryPopupMenu( mActiveItem )->popup( QCursor::pos() );
+          break;
+        default:
+          mCopyPopupMenu->datePicker()->setDate( date );
+          mCopyPopupMenu->datePicker()->setDate( QDate::currentDate() );
+          mItemPopupMenu->setItemEnabled( ePopupUnSubTodo,
+                                          mActiveItem->todo()->relatedTo() );
+          mItemPopupMenu->setItemEnabled( ePopupUnAllSubTodo,
+                                          !mActiveItem->todo()->relations().isEmpty() );
+          mItemPopupMenu->popup( QCursor::pos() );
+      }
+    } else {
+      mItemPopupMenu->popup( QCursor::pos() );
     }
   } else mPopupMenu->popup( QCursor::pos() );
 }
 
 void KOTodoView::newTodo()
 {
-  emit newTodoSignal();
+  emit newTodoSignal( QDate::currentDate().addDays(7) );
 }
 
 void KOTodoView::newSubTodo()
@@ -686,60 +841,148 @@ void KOTodoView::newSubTodo()
 
 void KOTodoView::editTodo()
 {
-  if (mActiveItem) {
-    emit editTodoSignal(mActiveItem->todo());
-  }
+  editItem( mActiveItem );
 }
 
 void KOTodoView::showTodo()
 {
-  if (mActiveItem) {
-    emit showTodoSignal(mActiveItem->todo());
-  }
+  showItem( mActiveItem );
+}
+
+void KOTodoView::printTodo()
+{
+#ifndef KORG_NOPRINTER
+  Calendar *cal;
+  KOCoreHelper helper;
+  CalPrinter printer( this, cal, &helper );
+  connect( this, SIGNAL(configChanged()), &printer, SLOT(updateConfig()) );
+
+  Incidence::List selectedIncidences;
+  selectedIncidences.append( mActiveItem->todo() );
+
+  printer.print( KOrg::CalPrinterBase::Incidence,
+                 QDate(), QDate(), selectedIncidences );
+#endif
 }
 
 void KOTodoView::deleteTodo()
 {
   if (mActiveItem) {
-    if (mActiveItem->childCount()) {
-      KMessageBox::sorry(this,i18n("Cannot delete To-Do which has children."),
-                         i18n("Delete To-Do"));
-    } else {
-      emit deleteTodoSignal(mActiveItem->todo());
-    }
+    emit deleteIncidenceSignal( mActiveItem->todo() );
   }
 }
 
 void KOTodoView::setNewPriority(int index)
 {
-  if (mActiveItem && !mActiveItem->todo()->isReadOnly ()) {
-    Todo*todo = mActiveItem->todo();
-    Todo*oldTodo = todo->clone();
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo->isReadOnly () &&
+       mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
     todo->setPriority(mPriority[index]);
     mActiveItem->construct();
-    emit todoModifiedSignal ( todo, oldTodo, KOGlobals::PRIORITY_MODIFIED );
+
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
   }
 }
 
-void KOTodoView::setNewPercentage(int index)
+void KOTodoView::setNewPercentage( KOTodoViewItem *item, int percentage )
 {
-  if ( mActiveItem && !mActiveItem->todo()->isReadOnly () ) {
-    Todo*todo = mActiveItem->todo();
-    Todo*oldTodo = todo->clone();
+  kdDebug(5850) << "KOTodoView::setNewPercentage( " << percentage << "), item = " << item << endl;
+  if ( !item || !mChanger  ) return;
+  Todo *todo = item->todo();
+  if ( !todo ) return;
 
-    if (mPercentage[index] == 100) {
-      todo->setCompleted(QDateTime::currentDateTime());
+  if ( !todo->isReadOnly () && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
+/*  Old code to make sub-items's percentage related to this one's:
+    QListViewItem *myChild = firstChild();
+    KOTodoViewItem *item;
+    while( myChild ) {
+      item = static_cast<KOTodoViewItem*>(myChild);
+      item->stateChange(state);
+      myChild = myChild->nextSibling();
+    }*/
+    if ( percentage == 100 ) {
+      todo->setCompleted( QDateTime::currentDateTime() );
+      // If the todo does recur, it doesn't get set as completed. However, the
+      // item is still checked. Uncheck it again.
+      if ( !todo->isCompleted() ) item->setState( QCheckListItem::Off );
+      else todo->setPercentComplete( percentage );
     } else {
-      todo->setCompleted(false);
+      todo->setCompleted( false );
+      todo->setPercentComplete( percentage );
     }
-    todo->setPercentComplete(mPercentage[index]);
-    mActiveItem->construct();
-    emit todoModifiedSignal( todo, oldTodo, KOGlobals::COMPLETION_MODIFIED );
+    item->construct();
+    if ( todo->doesRecur() && percentage == 100 )
+      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED_WITH_RECURRENCE );
+    else
+      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
+  } else {
+    item->construct();
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
   }
 }
 
+void KOTodoView::setNewPercentage( int index )
+{
+  setNewPercentage( mActiveItem, mPercentage[index] );
+}
+
+void KOTodoView::setNewDate( QDate date )
+{
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo ) return;
+
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
+    QDateTime dt;
+    dt.setDate( date );
+
+    if ( !todo->doesFloat() )
+      dt.setTime( todo->dtDue().time() );
+
+    if ( date.isNull() )
+      todo->setHasDueDate( false );
+    else if ( !todo->hasDueDate() )
+      todo->setHasDueDate( true );
+    todo->setDtDue( dt );
+
+    mActiveItem->construct();
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->endChange( todo );
+    delete oldTodo;
+  } else {
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
+  }
+}
+
+void KOTodoView::copyTodoToDate( QDate date )
+{
+  QDateTime dt( date );
+
+  if ( mActiveItem && mChanger ) {
+    Todo *newTodo = mActiveItem->todo()->clone();
+    newTodo->recreate();
+
+   newTodo->setHasDueDate( !date.isNull() );
+   newTodo->setDtDue( dt );
+   newTodo->setPercentComplete( 0 );
+
+   // avoid forking
+   if ( newTodo->doesRecur() )
+     newTodo->recurrence()->unsetRecurs();
+
+   mChanger->addIncidence( newTodo, this );
+ }
+}
 
 QPopupMenu *KOTodoView::getCategoryPopupMenu( KOTodoViewItem *todoItem )
 {
@@ -764,37 +1007,26 @@ QPopupMenu *KOTodoView::getCategoryPopupMenu( KOTodoViewItem *todoItem )
 
 void KOTodoView::changedCategories(int index)
 {
-  if (mActiveItem && !mActiveItem->todo()->isReadOnly ()) {
-    Todo*todo = mActiveItem->todo();
-    Todo*oldTodo = todo->clone();
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo ) return;
+
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
     QStringList categories = todo->categories ();
-    if (categories.find (mCategory[index]) != categories.end ())
-      categories.remove (mCategory[index]);
+    if ( categories.find( mCategory[index] ) != categories.end() )
+      categories.remove( mCategory[index] );
     else
-      categories.insert (categories.end(), mCategory[index]);
-    categories.sort ();
-    todo->setCategories (categories);
+      categories.insert( categories.end(), mCategory[index] );
+    categories.sort();
+    todo->setCategories( categories );
     mActiveItem->construct();
-    emit todoModifiedSignal( todo, oldTodo, KOGlobals::CATEGORY_MODIFIED);
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
-  }
-}
-
-void KOTodoView::itemClicked(QListViewItem *item)
-{
-  if (!item) return;
-
-  KOTodoViewItem *todoItem = (KOTodoViewItem *)item;
-  int completed = todoItem->todo()->isCompleted();  // Completed or not?
-
-  if (todoItem->isOn()) {
-    if (!completed) {
-      todoItem->todo()->setCompleted(QDateTime::currentDateTime());
-    }
   } else {
-    if (completed) {
-      todoItem->todo()->setCompleted(false);
-    }
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
   }
 }
 
@@ -816,6 +1048,22 @@ void KOTodoView::itemStateChanged( QListViewItem *item )
   if( mDocPrefs ) mDocPrefs->writeEntry( todoItem->todo()->uid(), todoItem->isOpen() );
 }
 
+void KOTodoView::setNewPercentageDelayed( KOTodoViewItem *item, int percentage )
+{
+  mPercentChangedMap.append( qMakePair( item, percentage ) );
+
+  QTimer::singleShot( 0, this, SLOT( processDelayedNewPercentage() ) );
+}
+
+void KOTodoView::processDelayedNewPercentage()
+{
+  QValueList< QPair< KOTodoViewItem *, int> >::Iterator it;
+  for ( it = mPercentChangedMap.begin(); it != mPercentChangedMap.end(); ++it )
+    setNewPercentage( (*it).first, (*it).second );
+
+  mPercentChangedMap.clear();
+}
+
 void KOTodoView::saveLayout(KConfig *config, const QString &group) const
 {
   mTodoListView->saveLayout(config,group);
@@ -840,11 +1088,6 @@ void KOTodoView::processSelectionChange()
   }
 }
 
-void KOTodoView::modified(bool b)
-{
-  emit isModified(b);
-}
-
 void KOTodoView::clearSelection()
 {
   mTodoListView->selectAll( false );
@@ -857,14 +1100,22 @@ void KOTodoView::purgeCompleted()
 
 void KOTodoView::addQuickTodo()
 {
-  Todo *todo = new Todo();
-  todo->setSummary( mQuickAdd->text() );
-  todo->setOrganizer( KOPrefs::instance()->email() );
-  if ( !calendar()->addTodo( todo ) ) {
-    KODialogManager::errorSaveTodo( this );
-    return;
+  if ( ! mQuickAdd->text().stripWhiteSpace().isEmpty() ) {
+    Todo *todo = new Todo();
+    todo->setSummary( mQuickAdd->text() );
+    todo->setOrganizer( Person( KOPrefs::instance()->fullName(),
+                        KOPrefs::instance()->email() ) );
+    if ( !mChanger->addIncidence( todo, this ) ) {
+      KODialogManager::errorSaveIncidence( this, todo );
+      delete todo;
+      return;
+    }
+    mQuickAdd->setText( QString::null );
   }
-  mQuickAdd->setText( QString::null );
-  emit todoAdded( todo );
-  updateView();
+}
+
+void KOTodoView::setIncidenceChanger( IncidenceChangerBase *changer )
+{
+  mChanger = changer;
+  mTodoListView->setIncidenceChanger( changer );
 }
