@@ -22,8 +22,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -33,6 +33,7 @@
 #include "plugin.h"
 
 class AbbrowserWidget;
+class KAboutData;
 
 class AbbrowserWidgetSetup : public ConduitConfigBase
 {
@@ -44,6 +45,7 @@ public:
 	static ConduitConfigBase *create(QWidget *,const char *);
 private:
 	AbbrowserWidget *fConfigWidget;
+	KAboutData *fAbout;
 } ;
 
 #endif
