@@ -101,7 +101,7 @@ DOCConduitFactory::~DOCConduitFactory()
 	}
 	if (qstrcmp(c, "SyncAction") == 0)
 	{
-		KPilotDeviceLink * d = dynamic_cast < KPilotDeviceLink * >(p);
+		KPilotLink * d = dynamic_cast < KPilotLink * >(p);
 		if (d)
 		{
 			return new DOCConduit(d, n, a);
@@ -109,7 +109,7 @@ DOCConduitFactory::~DOCConduitFactory()
 		else
 		{
 			kdError() << k_funcinfo
-				<<": Couldn't cast parent to KPilotDeviceLink" <<endl;
+				<<": Couldn't cast parent to KPilotLink" <<endl;
 			return 0L;
 		}
 	}
