@@ -1,8 +1,8 @@
-/* todo-conduit.cc  Todo-Conduit for syncing KPilot and KOrganizer
+/* Todo-Conduit for syncing KPilot and KOrganizer
 **
 ** Copyright (C) 2002-2003 Reinhold Kainhofer
 ** Copyright (C) 1998-2001 Dan Pilone
-** Copyright (C) 1998-2000 Preston Brown
+** Copyright (C) 1998-2000 Preston Brown <pbrown@kde.org>
 ** Copyright (C) 1998 Herwin-Jan Steehouwer
 ** Copyright (C) 2001 Cornelius Schumacher
 **
@@ -24,8 +24,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -52,10 +52,7 @@
 
 extern "C"
 {
-long version_conduit_todo = KPILOT_PLUGIN_API;
-
-const char *id_conduit_todo = "$Id: todo-conduit.cc,v 1.52.2.1 2004/09/29 09:20:54 adridg Exp $";
-
+unsigned long version_conduit_todo = Pilot::PLUGIN_API;
 }
 
 
@@ -82,7 +79,12 @@ int TodoConduitPrivate::updateIncidences()
 void TodoConduitPrivate::removeIncidence(KCal::Incidence *e)
 {
 	fAllTodos.remove(static_cast<KCal::Todo*>(e));
+	if (!fCalendar) return;
 	fCalendar->deleteTodo(static_cast<KCal::Todo*>(e));
+	// now just in case we're in the middle of reading through our list
+	// and we delete something, set reading to false so we start at the
+	// top again next time and don't have problems with our iterator
+	reading = false;
 }
 
 
@@ -100,7 +102,7 @@ KCal::Incidence *TodoConduitPrivate::findIncidence(recordid_t id)
 
 
 
-KCal::Incidence *TodoConduitPrivate::findIncidence(PilotAppCategory*tosearch)
+KCal::Incidence *TodoConduitPrivate::findIncidence(PilotRecordBase *tosearch)
 {
 	PilotTodoEntry*entry=dynamic_cast<PilotTodoEntry*>(tosearch);
 	if (!entry) return 0L;
@@ -123,12 +125,12 @@ KCal::Incidence *TodoConduitPrivate::getNextIncidence()
 	FUNCTIONSETUP;
 	if (reading) {
 		++fAllTodosIterator;
-	} 
+	}
 	else {
 		reading=true;
 		fAllTodosIterator = fAllTodos.begin();
 	}
-	
+
 	return(fAllTodosIterator == fAllTodos.end()) ? 0L : *fAllTodosIterator;
 }
 
@@ -142,14 +144,14 @@ KCal::Incidence *TodoConduitPrivate::getNextModifiedIncidence()
 	{
 		reading=true;
 		fAllTodosIterator = fAllTodos.begin();
-		if ( fAllTodosIterator != fAllTodos.end() ) e=*fAllTodosIterator;
 	}
 	else
 	{
 		++fAllTodosIterator;
 	}
+	if ( fAllTodosIterator != fAllTodos.end() ) e=*fAllTodosIterator;
 	while (fAllTodosIterator != fAllTodos.end() &&
-		e && e->syncStatus()!=KCal::Incidence::SYNCMOD)
+		e && e->syncStatus()!=KCal::Incidence::SYNCMOD && e->pilotId())
 	{
 		e = (++fAllTodosIterator != fAllTodos.end()) ? *fAllTodosIterator : 0L;
 
@@ -169,17 +171,13 @@ KCal::Incidence *TodoConduitPrivate::getNextModifiedIncidence()
  *                          TodoConduit class                               *
  ****************************************************************************/
 
-TodoConduit::TodoConduit(KPilotDeviceLink *d,
+TodoConduit::TodoConduit(KPilotLink *d,
 	const char *n,
-	const QStringList &a) : VCalConduitBase(d,n,a)
+	const QStringList &a) : VCalConduitBase(d,n,a),
+	fTodoAppInfo( 0L )
 {
 	FUNCTIONSETUP;
-#ifdef DEBUG
-	DEBUGCONDUIT << id_conduit_todo << endl;
-#endif
 	fConduitName=i18n("To-do");
-        
-        (void) id_conduit_todo;
 }
 
 
@@ -195,45 +193,40 @@ void TodoConduit::_setAppInfo()
 {
 	FUNCTIONSETUP;
 	// get the address application header information
-	int appLen = pack_ToDoAppInfo(&fTodoAppInfo, 0, 0);
-	unsigned char *buffer = new unsigned char[appLen];
-	pack_ToDoAppInfo(&fTodoAppInfo, buffer, appLen);
-	if (fDatabase) fDatabase->writeAppBlock(buffer, appLen);
-	if (fLocalDatabase) fLocalDatabase->writeAppBlock(buffer, appLen);
-	delete[] buffer;
+
+	if( !fTodoAppInfo )
+	{
+		DEBUGCONDUIT << fname << ": fTodoAppInfo is NULL" << endl;
+		return;
+	}
+	if( !fDatabase )
+	{
+		DEBUGCONDUIT << fname << ": fDatabase is NULL" << endl;
+		return;
+	}
+
+	fTodoAppInfo->writeTo(fDatabase);
 }
+
 void TodoConduit::_getAppInfo()
 {
 	FUNCTIONSETUP;
 	// get the address application header information
-	unsigned char *buffer =
-		new unsigned char[PilotRecord::APP_BUFFER_SIZE];
-	int appLen = fDatabase->readAppBlock(buffer,PilotRecord::APP_BUFFER_SIZE);
-
-	unpack_ToDoAppInfo(&fTodoAppInfo, buffer, appLen);
-	delete[]buffer;
-	buffer = NULL;
-
-#ifdef DEBUG
-	DEBUGCONDUIT << fname << " lastUniqueId"
-		<< fTodoAppInfo.category.lastUniqueID << endl;
-#endif
-	for (int i = 0; i < 16; i++)
-	{
-#ifdef DEBUG
-		DEBUGCONDUIT << fname << " cat " << i << " =" <<
-			fTodoAppInfo.category.name[i] << endl;
-#endif
-	}
 
+	KPILOT_DELETE( fTodoAppInfo );
+	fTodoAppInfo = new PilotToDoInfo(fDatabase);
+	fTodoAppInfo->dump();
 }
 
 
 
-const QString TodoConduit::getTitle(PilotAppCategory*de)
+const QString TodoConduit::getTitle(PilotRecordBase *de)
 {
 	PilotTodoEntry*d=dynamic_cast<PilotTodoEntry*>(de);
-	if (d) return QString(d->getDescription());
+	if (d)
+	{
+		return QString(d->getDescription());
+	}
 	return QString::null;
 }
 
@@ -243,17 +236,24 @@ void TodoConduit::readConfig()
 {
 	FUNCTIONSETUP;
 	VCalConduitBase::readConfig();
-	// determine if the categories have ever been synce. Needed to prevent loosing the categories on the desktop.
-	// also use a full sync for the first time to make sure the palm categories are really transferred to the desktop
+	// determine if the categories have ever been synce. Needed to prevent loosing
+	// the categories on the desktop. Also use a full sync for the first time to
+	// make sure the palm categories are really transferred to the desktop.
+	//
 	categoriesSynced = config()->conduitVersion()>=CONDUIT_VERSION_CATEGORYSYNC;
-	if (!categoriesSynced & !isFullSync() ) 
-		setSyncDirection(SyncAction::eFullSync);
-#ifdef DEBUG
-	DEBUGCONDUIT<<"categoriesSynced="<<categoriesSynced<<endl;
-#endif
+	if (!categoriesSynced && !isFullSync() )
+	{
+		changeSync(SyncMode::eFullSync);
+	}
+	DEBUGCONDUIT<<"categoriesSynced=" << categoriesSynced << endl;
 }
 
-
+void TodoConduit::preSync()
+{
+	FUNCTIONSETUP;
+	VCalConduitBase::preSync();
+	_getAppInfo();
+}
 
 void TodoConduit::postSync()
 {
@@ -267,7 +267,7 @@ void TodoConduit::postSync()
 
 
 
-PilotRecord*TodoConduit::recordFromIncidence(PilotAppCategory*de, const KCal::Incidence*e)
+PilotRecord*TodoConduit::recordFromIncidence(PilotRecordBase *de, const KCal::Incidence*e)
 {
 	// don't need to check for null pointers here, the recordFromIncidence(PTE*, KCal::Todo*) will do that.
 	PilotTodoEntry *tde = dynamic_cast<PilotTodoEntry*>(de);
@@ -282,14 +282,15 @@ PilotRecord*TodoConduit::recordFromTodo(PilotTodoEntry*de, const KCal::Todo*todo
 {
 	FUNCTIONSETUP;
 	if (!de || !todo) {
-#ifdef DEBUG
-		DEBUGCONDUIT<<fname<<": NULL todo given... Skipping it"<<endl;
-#endif
+		DEBUGCONDUIT << fname << ": NULL todo given... Skipping it" << endl;
 		return NULL;
 	}
 
 	// set secrecy, start/end times, alarms, recurrence, exceptions, summary and description:
-	if (todo->secrecy()!=KCal::Todo::SecrecyPublic) de->makeSecret();
+	if (todo->secrecy()!=KCal::Todo::SecrecyPublic)
+	{
+		de->setSecret( true );
+	}
 
 	// update it from the iCalendar Todo.
 
@@ -316,9 +317,7 @@ PilotRecord*TodoConduit::recordFromTodo(PilotTodoEntry*de, const KCal::Todo*todo
 	// what we call description pilot puts as a separate note
 	de->setNote(todo->description());
 
-#ifdef DEBUG
-DEBUGCONDUIT<<"-------- "<<todo->summary()<<endl;
-#endif
+	DEBUGCONDUIT << "-------- " << todo->summary() << endl;
 	return de->pack();
 }
 
@@ -329,7 +328,7 @@ void TodoConduit::preRecord(PilotRecord*r)
 	FUNCTIONSETUP;
 	if (!categoriesSynced && r)
 	{
-		const PilotAppCategory*de=newPilotEntry(r);
+		const PilotRecordBase *de=newPilotEntry(r);
 		KCal::Incidence *e = fP->findIncidence(r->id());
 		setCategory(dynamic_cast<KCal::Todo*>(e), dynamic_cast<const PilotTodoEntry*>(de));
 	}
@@ -339,26 +338,28 @@ void TodoConduit::preRecord(PilotRecord*r)
 
 void TodoConduit::setCategory(PilotTodoEntry*de, const KCal::Todo*todo)
 {
-	if (!de || !todo) return;
+	if (!de || !todo)
+	{
+		return;
+	}
 	de->setCategory(_getCat(todo->categories(), de->getCategoryLabel()));
 }
 
 
 
 /**
- * _getCat returns the id of the category from the given categories list. If none of the categories exist
- * on the palm, the "Nicht abgelegt" (don't know the english name) is used.
+ * _getCat returns the id of the category from the given categories list. If none of the
+ * categories exist on the palm, the "Unfiled" one is used.
  */
 QString TodoConduit::_getCat(const QStringList cats, const QString curr) const
 {
-	int j;
 	if (cats.size()<1) return QString::null;
 	if (cats.contains(curr)) return curr;
-	for ( QStringList::ConstIterator it = cats.begin(); it != cats.end(); ++it ) {
-		for (j=1; j<=15; j++)
+	for ( QStringList::ConstIterator it = cats.begin(); it != cats.end(); ++it )
+	{
+		for (unsigned int j=1; j<Pilot::CATEGORY_COUNT; j++)
 		{
-			QString catName = PilotAppCategory::codec()->
-				toUnicode(fTodoAppInfo.category.name[j]);
+			QString catName = fTodoAppInfo->categoryName(j);
 			if (!(*it).isEmpty() && !(*it).compare( catName ) )
 			{
 				return catName;
@@ -366,14 +367,16 @@ QString TodoConduit::_getCat(const QStringList cats, const QString curr) const
 		}
 	}
 	// If we have a free label, return the first possible cat
-	QString lastName(fTodoAppInfo.category.name[15]);
+	//
+	// FIXME: Clearly buggy, but I don't know how or why
+	QString lastName = fTodoAppInfo->categoryName(Pilot::CATEGORY_COUNT-1);
 	if (lastName.isEmpty()) return cats.first();
 	return QString::null;
 }
 
 
 
-KCal::Incidence *TodoConduit::incidenceFromRecord(KCal::Incidence *e, const PilotAppCategory *de)
+KCal::Incidence *TodoConduit::incidenceFromRecord(KCal::Incidence *e, const PilotRecordBase *de)
 {
 	return dynamic_cast<KCal::Incidence*>(incidenceFromRecord(dynamic_cast<KCal::Todo*>(e), dynamic_cast<const PilotTodoEntry*>(de)));
 }
@@ -396,6 +399,9 @@ KCal::Todo *TodoConduit::incidenceFromRecord(KCal::Todo *e, const PilotTodoEntry
    // We don't want this, do we?
 //	e->setOrganizer(fCalendar->getEmail());
 	e->setPilotId(de->id());
+#ifdef DEBUG
+		DEBUGCONDUIT<<fname<<": set KCal item to pilotId: [" << e->pilotId() << "] ..."<<endl;
+#endif
 	e->setSyncStatus(KCal::Incidence::SYNCNONE);
 	e->setSecrecy(de->isSecret() ? KCal::Todo::SecrecyPrivate : KCal::Todo::SecrecyPublic);
 
@@ -415,8 +421,6 @@ KCal::Todo *TodoConduit::incidenceFromRecord(KCal::Todo *e, const PilotTodoEntry
 	}
 
 	// Categories
-	// TODO: Sync categories
-	// first remove all categories and then add only the appropriate one
 	setCategory(e, de);
 
 	// PRIORITY //
@@ -442,19 +446,36 @@ void TodoConduit::setCategory(KCal::Todo *e, const PilotTodoEntry *de)
 {
 	if (!e || !de) return;
 	QStringList cats=e->categories();
-	int cat=de->getCat();
-	if (0<cat && cat<=15)
+	int cat=de->category();
+	if ( (0<cat) && (cat<(int)Pilot::CATEGORY_COUNT) )
 	{
-		QString newcat=PilotAppCategory::codec()->toUnicode(fTodoAppInfo.category.name[cat]);
+		QString newcat=fTodoAppInfo->categoryName(cat);
 		if (!cats.contains(newcat))
 		{
+			// if this event only has one category associated with it, then we can
+			// safely assume that what we should be doing here is changing it to match
+			// the palm.  if there's already more than one category in the event, however, we
+			// won't cause data loss--we'll just append what the palm has to the
+			// event's categories
+			if (cats.count() <=1) cats.clear();
+
 			cats.append( newcat );
 			e->setCategories(cats);
 		}
 	}
 }
 
-VCalConduitSettings *TodoConduit::config()
-{ 
-  return ToDoConduitFactory::config(); 
+static VCalConduitSettings *config_vcal = 0L;
+
+VCalConduitSettings *TodoConduit::theConfig() {
+	if (!config_vcal)
+	{
+		config_vcal = new VCalConduitSettings(CSL1("Calendar"));
+	}
+
+	return config_vcal;
+}
+
+VCalConduitSettings *TodoConduit::config() {
+	return theConfig();
 }
