@@ -19,16 +19,13 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
-static const char *todowidget_id =
-	"$Id: todoWidget.cc 444507 2005-08-10 12:59:25Z mlaurent $";
-
 
 #include "options.h"
 
@@ -79,8 +76,6 @@ TodoWidget::TodoWidget(QWidget * parent,
 	setupWidget();
 	fTodoList.setAutoDelete(true);
 
-	/* NOTREACHED */
-	(void) todowidget_id;
 }
 
 TodoWidget::~TodoWidget()
@@ -141,12 +136,10 @@ void TodoWidget::showComponent()
 #endif
 
 	fTodoDB = new PilotLocalDatabase(dbPath(), CSL1("ToDoDB"));
-	unsigned char buffer[PilotRecord::APP_BUFFER_SIZE];
-	int appLen;
 
 	fTodoList.clear();
 
-	if (fTodoDB->isDBOpen())
+	if (fTodoDB->isOpen())
 	{
 		KPILOT_DELETE(fTodoAppInfo);
 		fTodoAppInfo = new PilotToDoInfo(fTodoDB);
@@ -313,7 +306,7 @@ void TodoWidget::updateWidget()
 	{
 		todo=fTodoList.current();
 		if ((currentCatID == -1) ||
-			(todo->getCat() == currentCatID))
+			(todo->category() == currentCatID))
 		{
 			QString title = todo->getDescription();
 
@@ -399,7 +392,7 @@ void TodoWidget::slotCreateNewRecord()
 	//
 	PilotDatabase *myDB = new PilotLocalDatabase(dbPath(), CSL1("ToDoDB"));
 
-	if (!myDB || !myDB->isDBOpen())
+	if (!myDB || !myDB->isOpen())
 	{
 #ifdef DEBUG
 		DEBUGKPILOT << fname
@@ -409,7 +402,7 @@ void TodoWidget::slotCreateNewRecord()
 			<< " and got pointer @"
 			<< (void *) myDB
 			<< " with status "
-			<< ( myDB ? myDB->isDBOpen() : false )
+			<< ( myDB ? myDB->isOpen() : false )
 			<< endl;
 #endif
 
@@ -446,7 +439,7 @@ void TodoWidget::slotAddRecord(PilotTodoEntry * todo)
 		fTodoAppInfo->categoryInfo(), true);
 
 
-	todo->setCat(currentCatID);
+	todo->PilotRecordBase::setCategory(currentCatID);
 	fTodoList.append(todo);
 	writeTodo(todo);
 	// TODO: Just add the new record to the lists
@@ -510,10 +503,10 @@ void TodoWidget::slotDeleteRecord()
 
 	if (KMessageBox::questionYesNo(this,
 			i18n("Delete currently selected record?"),
-			i18n("Delete Record?")) == KMessageBox::No)
+			i18n("Delete Record?"), KStdGuiItem::del(), KStdGuiItem::cancel()) == KMessageBox::No)
 		return;
 
-	selectedRecord->makeDeleted();
+	selectedRecord->setDeleted(true);
 	writeTodo(selectedRecord);
 	emit(recordChanged(selectedRecord));
 	showComponent();
@@ -557,7 +550,7 @@ void TodoWidget::writeTodo(PilotTodoEntry * which,
 	PilotDatabase *myDB = todoDB;
 	bool usemyDB = false;
 
-	if (myDB == 0L || !myDB->isDBOpen())
+	if (myDB == 0L || !myDB->isOpen())
 	{
 		myDB = new PilotLocalDatabase(dbPath(), CSL1("ToDoDB"));
 		usemyDB = true;
@@ -566,7 +559,7 @@ void TodoWidget::writeTodo(PilotTodoEntry * which,
 	// Still no valid todo database...
 	//
 	//
-	if (!myDB->isDBOpen())
+	if (!myDB->isOpen())
 	{
 #ifdef DEBUG
 		DEBUGKPILOT << fname << ": Todo database is not open" <<
