@@ -26,8 +26,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -50,69 +50,28 @@
 #endif
 
 
-// Switch _on_ debugging if it's not off.
-//
-#ifndef NDEBUG
-#ifndef DEBUG
-#define DEBUG				(1)
-#endif
-#endif
-
-// Switch on debugging explicitly. Perhaps send debug to stderr instead
-// of to the KDE debugging facility (it does lose some niftiness then).
-//
-#ifndef DEBUG
-// #define DEBUG			(1)
-#endif
-// #define DEBUG_CERR			(1)
-
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <unistd.h>     /* For size_t for pilot-link */
 #include <qglobal.h>
-#include <pi-version.h>
-// For KDE_EXPORT with kdelibs 3.3.x
-#include <kdepimmacros.h>
 
-#if (QT_VERSION < 0x030200)
-#error "This is KPilot for KDE3.3 and won't compile with Qt < 3.2.0"
+#if (QT_VERSION < 0x030300)
+#error "This is KPilot for KDE3.5 and won't compile with Qt < 3.3.0"
 #endif
 
 #ifndef KDE_VERSION
 #include <kdeversion.h>
 #endif
 
-#if !(KDE_IS_VERSION(3,3,0))
-#error "This is KPilot for KDE 3.3 and won't compile with KDE < 3.3.0"
+#if !(KDE_IS_VERSION(3,4,0))
+#error "This is KPilot for (really) KDE 3.5 and won't compile with KDE < 3.4.0"
 #endif
 
-
-#ifndef PILOT_LINK_VERSION
-#error "You need at least pilot-link version 0.9.5"
+#if !(KDE_IS_VERSION(3,5,0))
+#warning "This is KPilot for KDE 3.5 and might not compile with KDE < 3.5.0"
 #endif
 
-
-#define PILOT_LINK_NUMBER	((10000*PILOT_LINK_VERSION) + \
-				(100*PILOT_LINK_MAJOR)+PILOT_LINK_MINOR)
-#define PILOT_LINK_0_10_0	(1000)
-#define PILOT_LINK_0_11_0	(1100)
-#define PILOT_LINK_0_11_8	(1108)
-#define PILOT_LINK_0_12_0	(1200)
-
-#if PILOT_LINK_NUMBER < PILOT_LINK_0_11_8
-#warning "You need at least pilot-link version 0.11.8 for modern devices"
-#endif
-
-#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
-#define PI_SIZE_T int
-#else
-#define PI_SIZE_T size_t
-#endif
-
-
-
+#include "pilotLinkVersion.h"
 
 // For QString, and everything else needs it anyway.
 #include <qstring.h>
@@ -122,13 +81,40 @@
 #ifdef DEBUG
 #undef NDEBUG
 #undef NO_DEBUG
+#ifndef DEBUG_CERR
+#define DEBUG_CERR
+#endif
 #endif
 #include <kdebug.h>
 
-
-
 extern KDE_EXPORT int debug_level;
 
+class KDE_EXPORT KPilotDepthCount
+{
+public:
+	KPilotDepthCount(int area, int level, const char *s);
+	~KPilotDepthCount();
+	QString indent() const;
+	const char *name() const { return fName; } ;
+	// if DEBUG_CERR is defined, we can't return std::cerr (by value),
+	// since the copy constructor is private!
+#ifndef DEBUG_CERR
+#ifdef NDEBUG
+	inline kndbgstream debug(int area=0)
+#else
+	inline kdbgstream debug(int area=0)
+#endif
+	{ return kdDebug(debug_level >= fLevel, area); }
+#endif
+
+protected:
+	static int depth;
+	int fDepth;
+	int fLevel;
+	const char *fName;
+} ;
+
+
 #ifdef DEBUG
 #ifdef __GNUC__
 #define KPILOT_FNAMEDEF(l)	KPilotDepthCount fname(DEBUGAREA,l,__FUNCTION__)
@@ -141,7 +127,7 @@ extern KDE_EXPORT int debug_level;
 #define DEBUGAREA 		0
 
 #define DEBUGAREA_KPILOT	5510
-#define DEBUGAREA_DAEMON	5511
+#define DEBUGAREA_LIBRARY	5511
 #define DEBUGAREA_CONDUIT	5512
 #define DEBUGAREA_DB		5513
 
@@ -149,37 +135,15 @@ extern KDE_EXPORT int debug_level;
 #include <iostream>
 #endif
 
-class KDE_EXPORT KPilotDepthCount
-{
-public:
-	KPilotDepthCount(int area, int level, const char *s);
-	~KPilotDepthCount();
-	QString indent() const;
-	const char *name() const { return fName; } ;
-	// if DEBUG_CERR is defined, we can't return std::cerr (by value),
-	// since the copy constructor is private!
-#ifndef DEBUG_CERR
-	inline kdbgstream debug(int area=0)
-	{ return kdDebug(debug_level >= fLevel, area); }
-#endif
-
-protected:
-	static int depth;
-	int fDepth;
-	int fLevel;
-	const char *fName;
-} ;
-
 // stderr / iostream-based debugging.
 //
 //
 #ifdef DEBUG_CERR
 #include <iostream>
 #define DEBUGKPILOT	std::cerr
-#define DEBUGDAEMON	std::cerr
+#define DEBUGLIBRARY	std::cerr
 #define DEBUGCONDUIT	std::cerr
 #define DEBUGDB		std::cerr
-
 using namespace std;
 
 inline std::ostream& operator <<(std::ostream &o, const QString &s)
@@ -194,11 +158,13 @@ inline std::ostream& operator <<(std::ostream &o, const KPilotDepthCount &d)
 
 #else
 
+#warning "kdDebug()-based debugging is deprecated"
+
 // kddebug based debugging
 //
 //
 #define DEBUGKPILOT	fname.debug(DEBUGAREA_KPILOT)
-#define DEBUGDAEMON	fname.debug(DEBUGAREA_DAEMON)
+#define DEBUGLIBRARY	fname.debug(DEBUGAREA_LIBRARY)
 #define DEBUGCONDUIT	fname.debug(DEBUGAREA_CONDUIT)
 #define DEBUGDB         fname.debug(DEBUGAREA_DB)
 
@@ -213,7 +179,7 @@ inline kdbgstream& operator <<(kdbgstream o, const KPilotDepthCount &d)
 #else
 #define DEBUGSTREAM	kndbgstream
 #define DEBUGKPILOT	kndDebug()
-#define DEBUGDAEMON	kndDebug()
+#define DEBUGLIBRARY	kndDebug()
 #define DEBUGCONDUIT	kndDebug()
 #define DEBUGDB         kndDebug()
 
@@ -224,7 +190,7 @@ inline kdbgstream& operator <<(kdbgstream o, const KPilotDepthCount &d)
 #define FUNCTIONSETUPL(a) const int fname = a; Q_UNUSED(fname);
 #endif
 
-#define KPILOT_VERSION	"4.5.4 (dreumes)"
+#define KPILOT_VERSION	"4.9.0 (deepsix)"
 
 
 // Function to expand newlines in rich text to <br>\n
