@@ -19,8 +19,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -28,17 +28,12 @@
 */
 
 
-static const char *pilotadress_id =
-	"$Id: pilotAddress.cc 412940 2005-05-12 23:32:07Z adridg $";
-
-#ifndef _KPILOT_OPTIONS_H
 #include "options.h"
-#endif
+
 
 #include <stdlib.h>
 #include <assert.h>
 
-#include <qtextcodec.h>
 #include <qstringlist.h>
 
 #include "pilotAddress.h"
@@ -46,45 +41,35 @@ static const char *pilotadress_id =
 
 #define MAXFIELDS 19
 
-PilotAddress::PilotAddress(struct AddressAppInfo &appInfo,
-	PilotRecord * rec) :
-	PilotAppCategory(rec),
-	fAppInfo(appInfo),
+PilotAddress::PilotAddress(PilotAddressInfo *info, PilotRecord *rec) :
+	PilotRecordBase(rec),
+	fAppInfo(*(info->info())),
 	fAddressInfo()
-{
-	FUNCTIONSETUPL(4);
-#if PILOT_LINK_NUMBER >= PILOT_LINK_0_12_0
-	pi_buffer_t b;
-	b.data = (unsigned char *) rec->getData();
-	b.allocated = b.used = rec->getLen();
-	if (rec) unpack_Address(&fAddressInfo, &b, address_v1);
-#else
-	if (rec) unpack_Address(&fAddressInfo, (unsigned char *) rec->getData(), rec->getLen());
-#endif
-	(void) pilotadress_id;
-	_loadMaps();
-}
-
-PilotAddress::PilotAddress(struct AddressAppInfo &appInfo) :
-	PilotAppCategory(),
-	fAppInfo(appInfo)
 {
 	FUNCTIONSETUPL(4);
 	reset();
 
-	// assign the phoneLabel so it doesn't appear in the pilot as
-	// work for all fields, but at least shows other fields
-	fAddressInfo.phoneLabel[0] = (int) eWork;
-	fAddressInfo.phoneLabel[1] = (int) eHome;
-	fAddressInfo.phoneLabel[2] = (int) eOther;
-	fAddressInfo.phoneLabel[3] = (int) eMobile;
-	fAddressInfo.phoneLabel[4] = (int) eEmail;
+	if (rec)
+	{
+		pi_buffer_t b;
+		b.data = (unsigned char *) rec->data();
+		b.allocated = b.used = rec->size();
+		unpack_Address(&fAddressInfo, &b, address_v1);
+	}
+	else
+	{
+		fAddressInfo.phoneLabel[0] = (int) eWork;
+		fAddressInfo.phoneLabel[1] = (int) eHome;
+		fAddressInfo.phoneLabel[2] = (int) eOther;
+		fAddressInfo.phoneLabel[3] = (int) eMobile;
+		fAddressInfo.phoneLabel[4] = (int) eEmail;
+	}
 
 	_loadMaps();
 }
 
 PilotAddress::PilotAddress(const PilotAddress & copyFrom) :
-	PilotAppCategory(copyFrom),
+	PilotRecordBase(copyFrom),
 	fAppInfo(copyFrom.fAppInfo),
 	fAddressInfo()
 {
@@ -97,7 +82,7 @@ PilotAddress::PilotAddress(const PilotAddress & copyFrom) :
 PilotAddress & PilotAddress::operator = (const PilotAddress & copyFrom)
 {
 	FUNCTIONSETUPL(4);
-	PilotAppCategory::operator = (copyFrom);
+	PilotRecordBase::operator = (copyFrom);
 	_copyAddressInfo(copyFrom.fAddressInfo);
 	return *this;
 }
@@ -148,7 +133,7 @@ PilotAddress::~PilotAddress()
 	free_Address(&fAddressInfo);
 }
 
-QString PilotAddress::getTextRepresentation(bool richText)
+QString PilotAddress::getTextRepresentation(bool richText) const
 {
 	QString text, tmp;
 
@@ -195,7 +180,7 @@ QString PilotAddress::getTextRepresentation(bool richText)
 			}
 			else
 				tmp=CSL1("%1: %2");
-			tmp=tmp.arg(PilotAppCategory::codec()->toUnicode(
+			tmp=tmp.arg(Pilot::fromPilot(
 				fAppInfo.phoneLabels[getPhoneLabelIndex(i-entryPhone1)]));
 			tmp=tmp.arg(rtExpand(getField(i), richText));
 			text += tmp;
@@ -264,8 +249,8 @@ QString PilotAddress::getTextRepresentation(bool richText)
 
 QString PilotAddress::getCategoryLabel() const
 {
-	int cat(getCat());
-	if (cat>0) return codec()->toUnicode(fAppInfo.category.name[cat]);
+	int cat(category());
+	if (cat>0) return Pilot::fromPilot(fAppInfo.category.name[cat]);
 	else return QString::null;
 }
 
@@ -288,10 +273,8 @@ QStringList PilotAddress::getEmails() const
 		}
 	}
 
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": returning: ["
-				<< list.size() << "] e-mail addresses." << endl;
-#endif
+	DEBUGLIBRARY << fname << ": returning: ["
+		<< list.size() << "] e-mail addresses." << endl;
 	return list;
 }
 
@@ -303,11 +286,10 @@ KABC::PhoneNumber::List PilotAddress::getPhoneNumbers() const
 	QString test;
 
 	int shownPhone = getShownPhone() + entryPhone1;
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": preferred pilot index is: ["
-				<< shownPhone << "], preferred phone number is: ["
-				<< getField(shownPhone) << "]" << endl;
-#endif
+
+	DEBUGLIBRARY << fname << ": preferred pilot index is: ["
+		<< shownPhone << "], preferred phone number is: ["
+		<< getField(shownPhone) << "]" << endl;
 
 	for (int i = entryPhone1; i <= entryPhone5; i++)
 	{
@@ -327,27 +309,24 @@ KABC::PhoneNumber::List PilotAddress::getPhoneNumbers() const
 					// if this is the preferred phone number, set it as such
 					if (shownPhone == i) {
 						phoneType |= KABC::PhoneNumber::Pref;
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": found preferred pilot index: ["
-				<< i << "], text: [" << test << "]" << endl;
-#endif
+						DEBUGLIBRARY << fname << ": found preferred pilot index: ["
+							<< i << "], text: [" << test << "]" << endl;
 					}
 					KABC::PhoneNumber ph(test, phoneType);
 					list.append(ph);
-				} else {
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": whoopsie.  pilot phone number: ["
-				<< test << "], index: [" << i << "], type: ["
-				<< ind << "], has no corresponding PhoneNumber type." << endl;
-#endif
+				}
+				else
+				{
+					DEBUGLIBRARY << fname << ": whoopsie.  pilot phone number: ["
+						<< test << "], index: [" << i << "], type: ["
+						<< ind << "], has no corresponding PhoneNumber type." << endl;
 				}
 			}
 		}
 	}
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": returning: ["
-				<< list.size() << "] phone numbers" << endl;
-#endif
+
+	DEBUGLIBRARY << fname << ": returning: ["
+		<< list.size() << "] phone numbers" << endl;
 	return list;
 }
 
@@ -388,13 +367,12 @@ void PilotAddress::setPhoneNumbers(KABC::PhoneNumber::List list)
 			int phoneKey = it.data();
 			if ( phone.type() & phoneKey)
 			{
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": found pilot type: ["
-				<< pilotKey << "] ("
-				<< fAppInfo.phoneLabels[pilotKey]
-				<< ") for PhoneNumber: ["
-				<< phone.number() << "]" << endl;
-#endif
+				DEBUGLIBRARY << fname << ": found pilot type: ["
+					<< pilotKey << "] ("
+					<< fAppInfo.phoneLabels[pilotKey]
+					<< ") for PhoneNumber: ["
+					<< phone.number() << "]" << endl;
+
 				category = pilotKey;
 				break;
 			}
@@ -403,32 +381,28 @@ void PilotAddress::setPhoneNumbers(KABC::PhoneNumber::List list)
 					  phone.number(), true, false);
 
 		// if this is the preferred phone number, then set it as such
-		if (phone.type() & KABC::PhoneNumber::Pref) {
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": found preferred PhoneNumber. "
+		if (phone.type() & KABC::PhoneNumber::Pref) 
+		{
+			DEBUGLIBRARY << fname << ": found preferred PhoneNumber. "
 				<< "setting showPhone to index: ["
 				<< fieldSlot << "], PhoneNumber: ["
 				<< phone.number() << "]" << endl;
-#endif
 			fAddressInfo.showPhone = fieldSlot - entryPhone1;
 		}
 	}
 
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": Pilot's showPhone now: ["
-				<< fAddressInfo.showPhone << "]." << endl;
-#endif
+	DEBUGLIBRARY << fname << ": Pilot's showPhone now: ["
+		<< fAddressInfo.showPhone << "]." << endl;
 
 	// after setting the numbers, make sure that something sensible is set as the
 	// shownPhone on the Pilot if nothing is yet...
 	QString pref = getField(fAddressInfo.showPhone + entryPhone1);
 	if (fAddressInfo.showPhone < 0 || fAddressInfo.showPhone > 4 || pref.isEmpty()) {
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": Pilot's showPhone: ["
-				<< fAddressInfo.showPhone
-				<< "] not properly set to a default. trying to set a sensible one."
-				<< endl;
-#endif
+		DEBUGLIBRARY << fname << ": Pilot's showPhone: ["
+			<< fAddressInfo.showPhone
+			<< "] not properly set to a default. trying to set a sensible one."
+			<< endl;
+
 		for (int i = entryPhone1; i <= entryPhone5; i++)
 		{
 			pref = getField(i);
@@ -439,10 +413,8 @@ void PilotAddress::setPhoneNumbers(KABC::PhoneNumber::List list)
 			}
 		}
 	}
-#ifdef DEBUG
-	DEBUGDAEMON << fname << ": Pilot's showPhone now: ["
-				<< fAddressInfo.showPhone << "], and that's final." << endl;
-#endif
+	DEBUGLIBRARY << fname << ": Pilot's showPhone now: ["
+		<< fAddressInfo.showPhone << "], and that's final." << endl;
 }
 
 void PilotAddress::setEmails(QStringList list)
@@ -513,7 +485,7 @@ void PilotAddress::_loadMaps()
 
 QString PilotAddress::getField(int field) const
 {
-	return codec()->toUnicode(fAddressInfo.entry[field]);
+	return Pilot::fromPilot(fAddressInfo.entry[field]);
 }
 
 int PilotAddress::_getNextEmptyPhoneSlot() const
@@ -551,7 +523,7 @@ int PilotAddress::setPhoneField(EPhoneType type, const QString &field,
 		{
 			QString custom4Field = getField(entryCustom4);
 			QString typeStr(
-				codec()->toUnicode(fAppInfo.phoneLabels[appPhoneLabelNum]));
+				Pilot::fromPilot(fAppInfo.phoneLabels[appPhoneLabelNum]));
 
 			custom4Field += typeStr + CSL1(" ") + fieldStr;
 			setField(entryCustom4, custom4Field);
@@ -597,7 +569,7 @@ QString PilotAddress::getPhoneField(EPhoneType type, bool checkCustom4) const
 		return QString::null;
 
 	// look for the phone type str
-	QString typeToStr(codec()->toUnicode(fAppInfo.phoneLabels[appTypeNum]));
+	QString typeToStr(Pilot::fromPilot(fAppInfo.phoneLabels[appTypeNum]));
 	QString customField(getField(entryCustom4));
 	int foundField = customField.find(typeToStr);
 
@@ -624,7 +596,7 @@ int PilotAddress::_getAppPhoneLabelNum(const QString & phoneType) const
 	FUNCTIONSETUPL(4);
 	for (int index = 0; index < 8; index++)
 	{
-		if (phoneType == codec()->toUnicode(fAppInfo.phoneLabels[index]))
+		if (phoneType == Pilot::fromPilot(fAppInfo.phoneLabels[index]))
 			return index;
 	}
 
@@ -662,7 +634,7 @@ void PilotAddress::setField(int field, const QString &text)
 	if (!text.isEmpty())
 	{
 		fAddressInfo.entry[field] = (char *) malloc(text.length() + 1);
-		strlcpy(fAddressInfo.entry[field], codec()->fromUnicode(text), text.length() + 1);
+		Pilot::toPilot(text, fAddressInfo.entry[field], text.length()+1);
 	}
 	else
 	{
@@ -670,20 +642,17 @@ void PilotAddress::setField(int field, const QString &text)
 	}
 }
 
-void *PilotAddress::pack_(void *buf, int *len)
+PilotRecord *PilotAddress::pack() const
 {
 	FUNCTIONSETUPL(4);
 	int i;
 
-#if PILOT_LINK_NUMBER >= PILOT_LINK_0_12_0
-	pi_buffer_t b = { 0,0,0 } ;
-	i = pack_Address(&fAddressInfo, &b, address_v1);
-	memcpy(buf,b.data,kMin(i,*len));
-	*len = kMin(i,*len);
-#else
-	i = pack_Address(&fAddressInfo, (unsigned char *) buf, *len);
-	*len = i;
-#endif
-	return buf;
+	pi_buffer_t *b = pi_buffer_new( sizeof(fAddressInfo) );
+	i = pack_Address(const_cast<Address_t *>(&fAddressInfo), b, address_v1);
+	if (i<0)
+	{
+		return 0L;
+	}
+	// pack_Address sets b->used
+	return new PilotRecord( b, this );
 }
-
