@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -67,7 +67,7 @@ static const char* incidenceInlineMimeType = "text/calendar";
 
 ResourceKolab::ResourceKolab( const KConfig *config )
   : ResourceCalendar( config ), ResourceKolabBase( "ResourceKolab-libkcal" ),
-    mOpen( false )
+    mCalendar( QString::fromLatin1("UTC") ), mOpen( false )
 {
   setType( "imap" );
   connect( &mResourceChangedTimer, SIGNAL( timeout() ),
@@ -116,6 +116,8 @@ bool ResourceKolab::doOpen()
   mOpen = true;
 
   KConfig config( configFile() );
+  config.setGroup( "General" );
+  mProgressDialogIncidenceLimit = config.readNumEntry("ProgressDialogIncidenceLimit", 200);
 
   return openResource( config, kmailCalendarContentsType, mEventSubResources )
     && openResource( config, kmailTodoContentsType, mTodoSubResources )
@@ -158,13 +160,15 @@ bool ResourceKolab::loadSubResource( const QString& subResource,
 
   const int nbMessages = 200; // read 200 mails at a time (see kabc resource)
 
-  const QString labelTxt = mimetype == "application/x-vnd.kolab.task" ? i18n( "Loading tasks..." )
-                           : mimetype == "application/x-vnd.kolab.journal" ? i18n( "Loading journals..." )
+  const QString labelTxt = !strcmp(mimetype, "application/x-vnd.kolab.task") ? i18n( "Loading tasks..." )
+                           : !strcmp(mimetype, "application/x-vnd.kolab.journal") ? i18n( "Loading journals..." )
                            : i18n( "Loading events..." );
-  (void)::Observer::self(); // ensure kio_uiserver is running
+  const bool useProgress = qApp && qApp->type() != QApplication::Tty && count > mProgressDialogIncidenceLimit;
+  if ( useProgress )
+    (void)::Observer::self(); // ensure kio_uiserver is running
   UIServer_stub uiserver( "kio_uiserver", "UIServer" );
   int progressId = 0;
-  if ( count > 200 ) {
+  if ( useProgress ) {
     progressId = uiserver.newJob( kapp->dcopClient()->appId(), true );
     uiserver.totalFiles( progressId, count );
     uiserver.infoMessage( progressId, labelTxt );
@@ -245,7 +249,10 @@ bool ResourceKolab::loadAllJournals()
 {
   removeIncidences( "Journal" );
   mCalendar.deleteAllJournals();
-  return doLoadAll( mJournalSubResources, journalAttachmentMimeType );
+  bool kolabStyle = doLoadAll( mJournalSubResources, journalAttachmentMimeType );
+  bool icalStyle = doLoadAll( mJournalSubResources, incidenceInlineMimeType );
+
+  return kolabStyle && icalStyle;
 }
 
 void ResourceKolab::removeIncidences( const QCString& incidenceType )
@@ -273,6 +280,11 @@ void ResourceKolab::removeIncidences( const QCString& incidenceType )
 bool ResourceKolab::doSave()
 {
   return true;
+  /*
+  return kmailTriggerSync( kmailCalendarContentsType )
+      && kmailTriggerSync( kmailTodoContentsType )
+      && kmailTriggerSync( kmailJournalContentsType );
+  */
 }
 
 void ResourceKolab::incidenceUpdated( KCal::IncidenceBase* incidencebase )
@@ -295,7 +307,7 @@ void ResourceKolab::incidenceUpdated( KCal::IncidenceBase* incidencebase )
   }
 
   QString subResource;
-  Q_UINT32 sernum;
+  Q_UINT32 sernum = 0;
   if ( mUidMap.contains( uid ) ) {
     subResource = mUidMap[ uid ].resource();
     sernum = mUidMap[ uid ].serialNumber();
@@ -342,7 +354,7 @@ void ResourceKolab::resolveConflict( KCal::Incidence* inc, const QString& subres
       mSilent = false;
       deleteIncidence( local ); // remove local from kmail
       kmailDeleteIncidence( subresource, sernum );// remove new from kmail
-      if ( localIncidence ) { 
+      if ( localIncidence ) {
         addIncidence( localIncidence, subresource, 0  );
         mUidsPendingAdding.remove( localIncidence->uid() ); // we do want to inform KOrg also
       }
@@ -438,6 +450,7 @@ bool ResourceKolab::addIncidence( KCal::Incidence* incidence, const QString& _su
   if ( !incidence ) return false;
   const QString &uid = incidence->uid();
   QString subResource = _subresource;
+
   Kolab::ResourceMap *map = &mEventSubResources; // don't use a ref here!
 
   const QString& type = incidence->type();
@@ -456,9 +469,12 @@ bool ResourceKolab::addIncidence( KCal::Incidence* incidence, const QString& _su
     if ( newIncidence ) {
       subResource = findWritableResource( *map );
     }
+
     if ( subResource.isEmpty() )
       return false;
 
+    mNewIncidencesMap.insert( uid, subResource );
+
     if ( !sendKMailUpdate( incidence, subResource, sernum ) ) {
       kdError(5650) << "Communication problem in ResourceKolab::addIncidence()\n";
       return false;
@@ -487,7 +503,7 @@ bool ResourceKolab::addIncidence( KCal::Incidence* incidence, const QString& _su
       mUidMap[ uid ] = StorageReference( subResource, sernum );
     } else {
       /* This is a real add, from KMail, we didn't trigger this ourselves.
-       * If this uid already exists in this folder, do conflict resolution, 
+       * If this uid already exists in this folder, do conflict resolution,
        * unless the folder is read-only, in which case the user should not be
        * offered a means of putting mails in a folder she'll later be unable to
        * upload. Skip the incidence, in this case. */
@@ -527,6 +543,7 @@ bool ResourceKolab::addIncidence( KCal::Incidence* incidence, const QString& _su
       }
     }
 
+    mNewIncidencesMap.remove( uid );
   }
   return true;
 }
@@ -534,7 +551,7 @@ bool ResourceKolab::addIncidence( KCal::Incidence* incidence, const QString& _su
 
 bool ResourceKolab::addEvent( KCal::Event* event )
 {
-  if ( mUidMap.contains( event->uid() ) ) 
+  if ( mUidMap.contains( event->uid() ) )
     return true; //noop
   else
     return addIncidence( event, QString::null, 0 );
@@ -570,9 +587,9 @@ bool ResourceKolab::deleteIncidence( KCal::Incidence* incidence )
   return true;
 }
 
-void ResourceKolab::deleteEvent( KCal::Event* event )
+bool ResourceKolab::deleteEvent( KCal::Event* event )
 {
-  deleteIncidence( event );
+  return deleteIncidence( event );
 }
 
 KCal::Event* ResourceKolab::event( const QString& uid )
@@ -586,9 +603,10 @@ KCal::Event::List ResourceKolab::rawEvents( EventSortField sortField, SortDirect
 }
 
 KCal::Event::List ResourceKolab::rawEventsForDate( const QDate& date,
-                                                   bool sorted )
+                                                   EventSortField sortField,
+                                                   SortDirection sortDirection )
 {
-  return mCalendar.rawEventsForDate( date, sorted );
+  return mCalendar.rawEventsForDate( date, sortField, sortDirection );
 }
 
 KCal::Event::List ResourceKolab::rawEventsForDate( const QDateTime& qdt )
@@ -605,7 +623,7 @@ KCal::Event::List ResourceKolab::rawEvents( const QDate& start,
 
 bool ResourceKolab::addTodo( KCal::Todo* todo )
 {
-  if ( mUidMap.contains( todo->uid() ) ) 
+  if ( mUidMap.contains( todo->uid() ) )
     return true; //noop
   else
     return addIncidence( todo, QString::null, 0 );
@@ -620,9 +638,9 @@ void ResourceKolab::addTodo( const QString& xml, const QString& subresource,
       addIncidence( todo, subresource, sernum );
 }
 
-void ResourceKolab::deleteTodo( KCal::Todo* todo )
+bool ResourceKolab::deleteTodo( KCal::Todo* todo )
 {
-  deleteIncidence( todo );
+  return deleteIncidence( todo );
 }
 
 KCal::Todo* ResourceKolab::todo( const QString& uid )
@@ -642,7 +660,7 @@ KCal::Todo::List ResourceKolab::rawTodosForDate( const QDate& date )
 
 bool ResourceKolab::addJournal( KCal::Journal* journal )
 {
-  if ( mUidMap.contains( journal->uid() ) ) 
+  if ( mUidMap.contains( journal->uid() ) )
     return true; //noop
   else
     return addIncidence( journal, QString::null, 0 );
@@ -659,9 +677,9 @@ void ResourceKolab::addJournal( const QString& xml, const QString& subresource,
   }
 }
 
-void ResourceKolab::deleteJournal( KCal::Journal* journal )
+bool ResourceKolab::deleteJournal( KCal::Journal* journal )
 {
-  deleteIncidence( journal );
+  return deleteIncidence( journal );
 }
 
 KCal::Journal* ResourceKolab::journal( const QString& uid )
@@ -868,6 +886,18 @@ ResourceKolab::labelForSubresource( const QString& subresource ) const
   return subresource;
 }
 
+QString ResourceKolab::subresourceIdentifier( Incidence *incidence )
+{
+  QString uid = incidence->uid();
+  if ( mUidMap.contains( uid ) )
+    return mUidMap[ uid ].resource();
+  else
+    if ( mNewIncidencesMap.contains( uid ) )
+      return mNewIncidencesMap[ uid ];
+    else
+      return QString();
+}
+
 void ResourceKolab::fromKMailAsyncLoadResult( const QMap<Q_UINT32, QString>& map,
                                               const QString& type,
                                               const QString& folder )
@@ -917,8 +947,8 @@ void ResourceKolab::setSubresourceActive( const QString &subresource, bool v )
 void ResourceKolab::slotEmitResourceChanged()
 {
    kdDebug(5650) << "KCal Kolab resource: emitting resource changed " << endl;
-   emit resourceChanged( this );
    mResourceChangedTimer.stop();
+   emit resourceChanged( this );
 }
 
 KABC::Lock* ResourceKolab::lock()
